#
# Copyright 2016.  Cumulus Networks, Inc.
#

import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors

class AcctonAS5610_52XQSFPp(cumulus.platform.Trident40GPort):
    def __init__(self, label, serdes, leds):
        cumulus.platform.Trident40GPort.__init__(self,
                                                 label=label,
                                                 serdes=serdes,
                                                 phy_types=None,
                                                 phy_addrs=None,
                                                 leds=leds,
                                                 unit=0)
        self.sfp = cumulus.sfp.SFPROMDTSLabel(label, 'qsfp+')
    _set_led = '''
;;; AcctonAS5610_52XQSFPp
;;;
;;; Two bits per port:
;;;
;;; 1st bit pushed on stack controls amber LED:
;;; 0 - off
;;; 1 - on
;;;
;;; 2n bit pushed on stack controls green LED:
;;; 0 - off
;;; 1 - on
;;;
;;; We will only use green LED.  1st bit pushed is always zero to
;;; disable amber LED.
set_led:
	port 1
	pushst PORTSTATUS_ZERO
	pack

        port    a
        ld      b,LINKSCAN_P
        add     b,a
        ld      b,(b)
        tst     b,LINKSCAN_TX
        push    cy
        tst     b,LINKSCAN_RX
        push    cy
        tor
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        pack

        ret
'''
    _set_led_4x10 = '''
;;;
;;; Input : register A, logical port
;;; Input / Output : register B, accumulated port status

;;; For each port accumulate 2 bits of information in register B:
;;;   register B[0] - activity (TX _OR_ RX)
;;;   register B[1] - link enabled

SUB_INTF_ACTIVITY	equ	0x0  ; activity
SUB_INTF_LINKEN		equ	0x1  ; link enabled

sub_intf_info:

        ;;; push OR of current TX/RX activity onto stack
        port a
        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor

        ;;; push accumlated link activity onto stack
        tst B, SUB_INTF_ACTIVITY
        push cy

        ;;; OR accumlated activity with current activity and store
        ;;; result back in register B
        tor
        pop
        bit B, SUB_INTF_ACTIVITY

        ;;; push current LINKEN onto stack
        pushst  PORTSTATUS_LINKEN

        ;;; push accumlated link enable onto stack
        tst B, SUB_INTF_LINKEN
        push cy

        ;;; OR accumlated LINKEN with current LINKEN and store
        ;;; result back in register B
        tor
        pop
        bit B, SUB_INTF_LINKEN

        ret

;;; AcctonAS5610_52XQSFPp_4x10
;;; OR together status of 4 ports for 4x10G QSFP port
;;; Input: register A - first logical port of 4-port group
;;; Modified: register B - used as scratch space
set_led:

        port 1
        pushst  PORTSTATUS_ZERO ; disable yellow LED
        pack

        ;;; initialize B
        ld	b,0

        ;;; load sub-0 info
        call	sub_intf_info

        ;;; load sub-1 info
        inc	a
        call	sub_intf_info

        ;;; load sub-2 info
        inc	a
        call	sub_intf_info

        ;;; load sub-3 info
        inc	a
        call	sub_intf_info

        ;;; push accumlated link activity onto stack
        tst	B, SUB_INTF_ACTIVITY
        push	cy

        ;;; combine with BLINK state
        push	(BLINK_STATE_P)
        tand
        tinv	; flash off with activity

        ;;; push accumlated link enable onto stack
        tst	B, SUB_INTF_LINKEN
        push	cy

        ;;; AND activity-blink state with accumlated LINKEN
        tand

        pack

        ret
'''

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        if (mode == "4x10G") and (sub != 0):
            # For 4x10G mode only update LEDs for the first
            # sub-interface.
            return set_calls

        fn_name = '%s' % self.__class__.__name__
        led_port = self.led_port_num(mode, 0)
        if mode == "4x10G":
            call = (fn_name + "_4x10", self._set_led_4x10, led_port)
        elif mode == "40G":
            call = (fn_name, self._set_led, led_port)

        set_calls[led_index_list[0]] = call
        set_calls[led_index_list[0] + 1] = None

        return set_calls

class AcctonAS5610_52XSFPp(cumulus.platform.Trident10GPort):
    _set_led = '''
;;; AcctonAS5610_52XSFPp
;;;
;;; Two bits per port:
;;;
;;; 1st bit pushed on stack controls amber LED:
;;; 0 - off
;;; 1 - on
;;;
;;; 2n bit pushed on stack controls green LED:
;;; 0 - off
;;; 1 - on
;;;
;;; We will only use green LED.  1st bit pushed is always zero to
;;; disable amber LED.
set_led:
	port 1
	pushst PORTSTATUS_ZERO
	pack

        port    a
        ld      b,LINKSCAN_P
        add     b,a
        ld      b,(b)
        tst     b,LINKSCAN_TX
        push    cy
        tst     b,LINKSCAN_RX
        push    cy
        tor
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        pack

        ret
'''

    def __init__(self, label, serdes, leds):
        cumulus.platform.Trident10GPort.__init__(self,
                                                 label=label,
                                                 serdes=serdes,
                                                 phy_type=None,
                                                 phy_addr=None,
                                                 phy_layer='SFP+',
                                                 leds=leds,
                                                 unit=0)
        self.sfp = cumulus.sfp.SFPROMDTSLabel(label, 'sfp+')

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        fn_name = '%s' % self.__class__.__name__
        call = (fn_name, self._set_led, self.hw_port_num(mode, 0))
        for led in led_index_list:
            set_calls[led] = call
            set_calls[led + 1] = None

        return set_calls

class AcctonAS5610_52XCPLD(cumulus.cpld.SysFS):
    path = '/sys/devices/ff705000.localbus/ea000000.cpld'

class AcctonAS5610_52XLED(cumulus.ledup.TridentLEDController):
    num_leds = 64

class AcctonAS5610_52XSwitch(cumulus.platform.BCMSwitch):
    ledup = AcctonAS5610_52XLED
    _sfp = AcctonAS5610_52XSFPp
    _qsfp = AcctonAS5610_52XQSFPp
    has_phy_ucode_mdio = False
    # This board does not have SPI-ROMs.
    has_phy_ucode_spi_rom = False
    ports = (
        _sfp(label="1",  serdes=64, leds=(1, 34)),
        _sfp(label="2",  serdes=65, leds=(1, 32)),
        _sfp(label="3",  serdes=66, leds=(1, 38)),
        _sfp(label="4",  serdes=67, leds=(1, 36)),
        _sfp(label="5",  serdes=68, leds=(1, 62)),
        _sfp(label="6",  serdes=69, leds=(1, 60)),
        _sfp(label="7",  serdes=70, leds=(1, 58)),
        _sfp(label="8",  serdes=71, leds=(1, 56)),
        _sfp(label="9",  serdes=4,  leds=(0, 2)),
        _sfp(label="10", serdes=5,  leds=(0, 0)),
        _sfp(label="11", serdes=6,  leds=(0, 6)),
        _sfp(label="12", serdes=7,  leds=(0, 4)),
        _sfp(label="13", serdes=8,  leds=(0, 50)),
        _sfp(label="14", serdes=9,  leds=(0, 48)),
        _sfp(label="15", serdes=10, leds=(0, 54)),
        _sfp(label="16", serdes=11, leds=(0, 52)),
        _sfp(label="17", serdes=12, leds=(0, 46)),
        _sfp(label="18", serdes=13, leds=(0, 44)),
        _sfp(label="19", serdes=14, leds=(0, 42)),
        _sfp(label="20", serdes=15, leds=(0, 40)),
        _sfp(label="21", serdes=17, leds=(0, 62)),
        _sfp(label="22", serdes=16, leds=(0, 60)),
        _sfp(label="23", serdes=19, leds=(0, 58)),
        _sfp(label="24", serdes=18, leds=(0, 56)),
        _sfp(label="25", serdes=21, leds=(0, 38)),
        _sfp(label="26", serdes=20, leds=(0, 36)),
        _sfp(label="27", serdes=23, leds=(0, 34)),
        _sfp(label="28", serdes=22, leds=(0, 32)),
        _sfp(label="29", serdes=24, leds=(0, 30)),
        _sfp(label="30", serdes=25, leds=(0, 28)),
        _sfp(label="31", serdes=26, leds=(0, 26)),
        _sfp(label="32", serdes=27, leds=(0, 24)),
        _sfp(label="33", serdes=28, leds=(0, 14)),
        _sfp(label="34", serdes=29, leds=(0, 8)),
        _sfp(label="35", serdes=30, leds=(0, 10)),
        _sfp(label="36", serdes=31, leds=(0, 12)),
        _sfp(label="37", serdes=32, leds=(0, 22)),
        _sfp(label="38", serdes=33, leds=(0, 20)),
        _sfp(label="39", serdes=34, leds=(0, 18)),
        _sfp(label="40", serdes=35, leds=(0, 16)),
        _sfp(label="41", serdes=36, leds=(1, 44)),
        _sfp(label="42", serdes=37, leds=(1, 42)),
        _sfp(label="43", serdes=38, leds=(1, 40)),
        _sfp(label="44", serdes=39, leds=(1, 46)),
        _sfp(label="45", serdes=40, leds=(1, 52)),
        _sfp(label="46", serdes=41, leds=(1, 50)),
        _sfp(label="47", serdes=42, leds=(1, 48)),
        _sfp(label="48", serdes=43, leds=(1, 54)),

        _qsfp(label="49", serdes=(48,49,50,51), leds=(1, 26)),
        _qsfp(label="50", serdes=(44,45,46,47), leds=(1, 24)),
        _qsfp(label="51", serdes=(60,61,62,63), leds=(1, 30)),
        _qsfp(label="52", serdes=(56,57,58,59), leds=(1, 28)),
    )

class AcctonAS5610_52xPSU1(cumulus.sensors.PSU_Unit):
    name = 'PSU1'
    cpld_hwmon = ['psu_pwr1']

class AcctonAS5610_52xPSU2(cumulus.sensors.PSU_Unit):
    name = 'PSU2'
    cpld_hwmon = ['psu_pwr2']

class AcctonAS5610_52xTemp1(cumulus.sensors.Temp_Unit):
    name = 'Temp1'
    description = cumulus.sensors.Temp_Unit.mac_die_sensor_name
    driver_path = '/sys/devices/pci0000:00/0000:00:00.0/0000:01:00.0'
    driver_hwmon = ['temp1']
    temp_fan_min = 65
    temp_max = 100
    temp_crit = 105

class AcctonAS5610_52BoardTemp(cumulus.sensors.Temp_Unit):
    def __init__(self):
        self.name =  (self.__class__.__name__[16:])
        self.temp_fan_min = 55
        self.temp_max = 75
        self.temp_crit = 85
        self.description = 'Board Temperature Sensor'
        self.driver_path = '/sys/devices/soc.0/ff703000.i2c/i2c-0/i2c-9/9-004d'
        cumulus.sensors.Temp_Unit.__init__(self)

class AcctonAS5610_52xTemp2(AcctonAS5610_52BoardTemp):
    driver_hwmon = ['temp1']

class AcctonAS5610_52xTemp3(AcctonAS5610_52BoardTemp):
    driver_hwmon = ['temp2']

class AcctonAS5610_52xTemp4(AcctonAS5610_52BoardTemp):
    driver_hwmon = ['temp3']

class AcctonAS5610_52xTemp5(AcctonAS5610_52BoardTemp):
    driver_hwmon = ['temp4']

class AcctonAS5610_52xTemp6(AcctonAS5610_52BoardTemp):
    driver_hwmon = ['temp5']

class AcctonAS5610_52xTemp7(AcctonAS5610_52BoardTemp):
    driver_hwmon = ['temp6']

class AcctonAS5610_52xTemp8(AcctonAS5610_52BoardTemp):
    driver_hwmon = ['temp7']

class AcctonAS5610_52xTemp9(cumulus.sensors.Temp_Unit):
    name = 'Temp9'
    description = 'NE1617A Sensor Temperature'
    driver_path = '/sys/devices/soc.0/ff703000.i2c/i2c-0/i2c-9/9-0018'
    driver_hwmon = ['temp1']
    temp_fan_min = 55
    temp_max = 75
    temp_crit = 85

class AcctonAS5610_52xTemp10(cumulus.sensors.Temp_Unit):
    name = 'Temp10'
    description = cumulus.sensors.Temp_Unit.mac_board_sensor_name
    driver_path = '/sys/devices/soc.0/ff703000.i2c/i2c-0/i2c-9/9-0018'
    driver_hwmon = ['temp2']
    temp_fan_min = 65
    temp_max = 80
    temp_crit = 90

class AcctonAS5610_52xFan(cumulus.sensors.Fan_Unit):
    name = 'Fan'
    description = 'System Fan'
    cpld_hwmon = ['system_fan']
    pwm_path = '/sys/devices/ff705000.localbus/ea000000.cpld'

class AcctonAS5610_52xPSU1LED(cumulus.sensors.StatusLED_Unit):
    name = 'PSU1'
    sources = ['PSU1']
    led_location = 'led_psu1'

class AcctonAS5610_52xPSU2LED(cumulus.sensors.StatusLED_Unit):
    name = 'PSU2'
    sources = ['PSU2']
    led_location = 'led_psu2'

class AcctonAS5610_52xFanLED(cumulus.sensors.StatusLED_Unit):
    name = 'Fan'
    sources = ['Fan']
    led_location = 'led_fan'

class AcctonAS5610_52xSystemLED(cumulus.sensors.StatusLED_Unit):
    name = 'System'
    led_location = 'led_diag'

class AcctonAS5610_52xSensors():
    units = ( AcctonAS5610_52xTemp1(),
              AcctonAS5610_52xTemp2(),
              AcctonAS5610_52xTemp3(),
              AcctonAS5610_52xTemp4(),
              AcctonAS5610_52xTemp5(),
              AcctonAS5610_52xTemp6(),
              AcctonAS5610_52xTemp7(),
              AcctonAS5610_52xTemp8(),
              AcctonAS5610_52xTemp9(),
              AcctonAS5610_52xTemp10(),
              AcctonAS5610_52xPSU1(),
              AcctonAS5610_52xPSU2(),
              AcctonAS5610_52xFan(),
        )

class AcctonAS5610_52xPWM1(cumulus.sensors.PWM_Unit):
    fans = (AcctonAS5610_52xFan(),
            )
    temps = ( AcctonAS5610_52xTemp1(),
              AcctonAS5610_52xTemp2(),
              AcctonAS5610_52xTemp3(),
              AcctonAS5610_52xTemp4(),
              AcctonAS5610_52xTemp5(),
              AcctonAS5610_52xTemp6(),
              AcctonAS5610_52xTemp7(),
              AcctonAS5610_52xTemp8(),
              AcctonAS5610_52xTemp9(),
              AcctonAS5610_52xTemp10(),
              )

class AcctonAS5610_52xPWMs():
    units = (AcctonAS5610_52xPWM1(),
             )

class AcctonAS5610_52xLEDs():
    units = (AcctonAS5610_52xPSU1LED(),
             AcctonAS5610_52xPSU2LED(),
             AcctonAS5610_52xFanLED(),
             AcctonAS5610_52xSystemLED(),
             )

class AcctonAS5610_52X(cumulus.platform.Platform):
    name = 'accton,as5610_52x'
    cpld = AcctonAS5610_52XCPLD()
    chip = cumulus.platform.TridentPlusChip()
    switch = AcctonAS5610_52XSwitch(chip)
    sensors = AcctonAS5610_52xSensors()
    pwms = AcctonAS5610_52xPWMs()
    statusleds = AcctonAS5610_52xLEDs()

Platform = AcctonAS5610_52X
