#
# Copyright 2016, 2017.  Cumulus Networks, Inc.
#

import subprocess
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors
import cumulus.platforms.mlnx_SN2xxx

AcctonAS7412_32X_cpld_path = '/sys/bus/platform/devices/act_as7412_32x_cpld.0'

class AcctonAS7412_32XCPLD(cumulus.cpld.SysFS):
    path = AcctonAS7412_32X_cpld_path

class AcctonAS7412_32XFanOdd(cumulus.sensors.Fan_Unit):
    fan_min = 7600
    fan_max = 22000
    driver_path = AcctonAS7412_32X_cpld_path
    pwm_path = driver_path

class AcctonAS7412_32XFanEven(AcctonAS7412_32XFanOdd):
    fan_min = 6500
    fan_max = 17500

class AcctonAS7412_32XFan1(AcctonAS7412_32XFanOdd):
    name = 'Fan1'
    description = 'Fan Tray 1'
    cpld_hwmon = ['fan1_ok']
    pwm_hwmon = ['pwm1']

class AcctonAS7412_32XFan2(AcctonAS7412_32XFanEven):
    name = 'Fan2'
    description = 'Fan Tray 1'
    cpld_hwmon = ['fan2_ok']
    driver_hwmon = ['fan2']
    pwm_hwmon = ['pwm1']

class AcctonAS7412_32XFan3(AcctonAS7412_32XFanOdd):
    name = 'Fan3'
    description = 'Fan Tray 2'
    cpld_hwmon = ['fan3_ok']
    driver_hwmon = ['fan3']
    pwm_hwmon = ['pwm2']

class AcctonAS7412_32XFan4(AcctonAS7412_32XFanEven):
    name = 'Fan4'
    description = 'Fan Tray 2'
    cpld_hwmon = ['fan4_ok']
    driver_hwmon = ['fan4']
    pwm_hwmon = ['pwm2']

class AcctonAS7412_32XFan5(AcctonAS7412_32XFanOdd):
    name = 'Fan5'
    description = 'Fan Tray 3'
    cpld_hwmon = ['fan5_ok']
    driver_hwmon = ['fan5']
    pwm_hwmon = ['pwm3']

class AcctonAS7412_32XFan6(AcctonAS7412_32XFanEven):
    name = 'Fan6'
    description = 'Fan Tray 3'
    cpld_hwmon = ['fan6_ok']
    driver_hwmon = ['fan6']
    pwm_hwmon = ['pwm3']

class AcctonAS7412_32XFan7(AcctonAS7412_32XFanOdd):
    name = 'Fan7'
    description = 'Fan Tray 4'
    cpld_hwmon = ['fan7_ok']
    driver_hwmon = ['fan7']
    pwm_hwmon = ['pwm4']

class AcctonAS7412_32XFan8(AcctonAS7412_32XFanEven):
    name = 'Fan8'
    description = 'Fan Tray 4'
    cpld_hwmon = ['fan8_ok']
    driver_hwmon = ['fan8']
    pwm_hwmon = ['pwm4']

class AcctonAS7412_32XFan9(AcctonAS7412_32XFanOdd):
    name = 'Fan9'
    description = 'Fan Tray 5'
    cpld_hwmon = ['fan9_ok']
    driver_hwmon = ['fan9']
    pwm_hwmon = ['pwm5']

class AcctonAS7412_32XFan10(AcctonAS7412_32XFanEven):
    name = 'Fan10'
    description = 'Fan Tray 5'
    cpld_hwmon = ['fan10_ok']
    driver_hwmon = ['fan10']
    pwm_hwmon = ['pwm5']

class AcctonAS7412_32XFan11(AcctonAS7412_32XFanOdd):
    name = 'Fan11'
    description = 'Fan Tray 6'
    cpld_hwmon = ['fan11_ok']
    driver_hwmon = ['fan11']
    pwm_hwmon = ['pwm6']

class AcctonAS7412_32XFan12(AcctonAS7412_32XFanEven):
    name = 'Fan12'
    description = 'Fan Tray 6'
    cpld_hwmon = ['fan12_ok']
    driver_hwmon = ['fan12']
    pwm_hwmon = ['pwm6']

class AcctonAS7412_32XTemp1(cumulus.sensors.Temp_Unit):
    name = 'Temp1'
    description = 'Power VCORE'
    driver_path = '/sys/bus/i2c/devices/11-0048'
    temp_max  = 60
    temp_crit = 70

class AcctonAS7412_32XTemp2(cumulus.sensors.Temp_Unit):
    name = 'Temp2'
    description = 'Switch Board'
    driver_path = '/sys/bus/i2c/devices/11-0049'
    temp_max  = 60
    temp_crit = 70

class AcctonAS7412_32XTemp3(cumulus.sensors.Temp_Unit):
    name = 'Temp3'
    description = 'Ambient'
    driver_path = '/sys/bus/i2c/devices/11-004a'
    temp_max  = 60
    temp_crit = 70

class AcctonAS7412_32XTemp4(cumulus.sensors.Temp_Unit):
    name = 'Temp4'
    description = 'CPU'
    driver_path = '/sys/bus/i2c/devices/11-004b'
    temp_max  = 60
    temp_crit = 70

class AcctonAS7412_32X_PSU(cumulus.sensors.PSU_Unit):
    init_done = False
    def __init__(self):
        cumulus.sensors.PSU_Unit.__init__(self)
        self.add_state('OK', self.ok_handler)

    def ok_handler(self):
        if self.init_done:
            return
        err_states = ['BAD', 'ABSENT'];
        if self.attrs['prev_state'] in err_states:
            cmd = ['/sbin/rmmod', 'pmbus']
            retrm =  subprocess.call(cmd)
            if retrm == 0:
                cmd = ['/sbin/modprobe', 'pmbus']
                subprocess.call(cmd)
            self.init_done = True

class AcctonAS7412_32XPSU1(AcctonAS7412_32X_PSU):
    name = 'PSU1'
    cpld_hwmon = ['psu_pwr1']

class AcctonAS7412_32XPSU2(AcctonAS7412_32X_PSU):
    name = 'PSU2'
    cpld_hwmon = ['psu_pwr2']

AcctonAS7412_32X_psu1_path = '/sys/bus/i2c/devices/21-005b'
AcctonAS7412_32X_psu2_path = '/sys/bus/i2c/devices/20-0058'
class AcctonAS7412_32XPSUTemp(cumulus.sensors.Temp_Unit):
    driver_hwmon = ['temp1']
    cpld_path = AcctonAS7412_32X_cpld_path
    temp_max = 60
    temp_crit = 100

    def find_and_add_attrs(self):
        if self.get_cpld_value(self.cpld_psu).find('ok') >= 0:
            return cumulus.sensors.Temp_Unit.find_and_add_attrs(self)
        else:
            return -1

class AcctonAS7412_32XTemp5(AcctonAS7412_32XPSUTemp):
    name = 'PSU1Temp1'
    description = 'PSU1 Temp1'
    driver_hwmon = ['temp1']
    driver_path = AcctonAS7412_32X_psu1_path
    cpld_psu = 'psu_pwr1'

class AcctonAS7412_32XTemp6(AcctonAS7412_32XPSUTemp):
    name = 'PSU1Temp2'
    description = 'PSU1 Temp2'
    driver_hwmon = ['temp2']
    driver_path = AcctonAS7412_32X_psu1_path
    cpld_psu = 'psu_pwr1'

class AcctonAS7412_32XTemp7(AcctonAS7412_32XPSUTemp):
    name = 'PSU1Temp3'
    description = 'PSU1 Temp3'
    driver_hwmon = ['temp3']
    driver_path = AcctonAS7412_32X_psu1_path
    cpld_psu = 'psu_pwr1'

class AcctonAS7412_32XTemp8(AcctonAS7412_32XPSUTemp):
    name = 'PSU2Temp1'
    description = 'PSU2 Temp1'
    driver_hwmon = ['temp1']
    driver_path = AcctonAS7412_32X_psu2_path
    cpld_psu = 'psu_pwr2'

class AcctonAS7412_32XTemp9(AcctonAS7412_32XPSUTemp):
    name = 'PSU2Temp2'
    description = 'PSU2 Temp2'
    driver_hwmon = ['temp2']
    driver_path = AcctonAS7412_32X_psu2_path
    cpld_psu = 'psu_pwr2'

class AcctonAS7412_32XTemp10(AcctonAS7412_32XPSUTemp):
    name = 'PSU2Temp3'
    description = 'PSU2 Temp3'
    driver_hwmon = ['temp3']
    driver_path = AcctonAS7412_32X_psu2_path
    cpld_psu = 'psu_pwr2'

class AcctonAS7412_32XPSUFan(cumulus.sensors.Fan_Unit):
    driver_hwmon = ['fan1']
    pwm_hwmon = ['pwm1']
    cpld_path = AcctonAS7412_32X_cpld_path
    fan_max = 24500
    fan_min = 5000

    def find_and_add_attrs(self):
        if self.get_cpld_value(self.cpld_hwmon[0]).find('ok') >= 0:
            return cumulus.sensors.Fan_Unit.find_and_add_attrs(self)
        else:
            return -1

class AcctonAS7412_32XFan13(AcctonAS7412_32XPSUFan):
    name = 'PSU1Fan1'
    description = 'PSU1 Fan'
    driver_path = AcctonAS7412_32X_psu1_path
    cpld_hwmon = ['psu_pwr1']

class AcctonAS7412_32XFan14(AcctonAS7412_32XPSUFan):
    name = 'PSU2Fan1'
    description = 'PSU2 Fan'
    driver_path = AcctonAS7412_32X_psu2_path
    cpld_hwmon = ['psu_pwr2']

class AcctonAS7412QSFP28(cumulus.platform.Spectrum100GPort):
    _set_led = '''
;;; AcctonAS7412QSFP28
;;; LEDs handled by hardware
'''
    def __init__(self, label, adj_label, phy_addrs, hw_ports,
                 blocked_adj_split=None):

        hw_port_names = []
        for hw_port in hw_ports:
            hw_port_names.append('xe%d' % hw_port)

        cumulus.platform.Spectrum100GPort.__init__(self, label=label,
                        adj_label=adj_label, hw_ports=tuple(hw_port_names),
                        phy_addrs=phy_addrs, blocked_adj_split=blocked_adj_split)
        self.sfp = cumulus.sfp.SFPROMSDK(hw_ports[0], 'qsfp28')

    def set_leds(self, mode, led_port, sub):
        self._check_mode(mode)

        set_calls = {}

        return set_calls

class AcctonAS7412_32XSensors():
    units = (
        AcctonAS7412_32XFan1(),
        AcctonAS7412_32XFan2(),
        AcctonAS7412_32XFan3(),
        AcctonAS7412_32XFan4(),
        AcctonAS7412_32XFan5(),
        AcctonAS7412_32XFan6(),
        AcctonAS7412_32XFan7(),
        AcctonAS7412_32XFan8(),
        AcctonAS7412_32XFan9(),
        AcctonAS7412_32XFan10(),
        AcctonAS7412_32XFan11(),
        AcctonAS7412_32XFan12(),
        AcctonAS7412_32XFan13(),
        AcctonAS7412_32XFan14(),
        AcctonAS7412_32XTemp1(),
        AcctonAS7412_32XTemp2(),
        AcctonAS7412_32XTemp3(),
        AcctonAS7412_32XTemp4(),
        AcctonAS7412_32XTemp5(),
        AcctonAS7412_32XTemp6(),
        AcctonAS7412_32XTemp7(),
        AcctonAS7412_32XTemp8(),
        AcctonAS7412_32XTemp9(),
        AcctonAS7412_32XTemp10(),
        AcctonAS7412_32XPSU1(),
        AcctonAS7412_32XPSU2(),
    )

class AcctonAS7412_32XPWM1(cumulus.sensors.PWM_Unit):
    name = 'System PWM'
    fans = (
        AcctonAS7412_32XFan1(),
        AcctonAS7412_32XFan2(),
        AcctonAS7412_32XFan3(),
        AcctonAS7412_32XFan4(),
        AcctonAS7412_32XFan5(),
        AcctonAS7412_32XFan6(),
        AcctonAS7412_32XFan7(),
        AcctonAS7412_32XFan8(),
        AcctonAS7412_32XFan9(),
        AcctonAS7412_32XFan10(),
        AcctonAS7412_32XFan11(),
        AcctonAS7412_32XFan12(),
    )
    temps = (
        AcctonAS7412_32XTemp1(),
        AcctonAS7412_32XTemp2(),
        AcctonAS7412_32XTemp3(),
        AcctonAS7412_32XTemp4(),
    )

class AcctonAS7412_32XPWM2(cumulus.sensors.PWM_Unit):
    name = 'PSU1 PWM'
    fans = (
        AcctonAS7412_32XFan13(),
    )
    temps = (
        AcctonAS7412_32XTemp5(),
        AcctonAS7412_32XTemp6(),
        AcctonAS7412_32XTemp7(),
    )

class AcctonAS7412_32XPWM3(cumulus.sensors.PWM_Unit):
    name = 'PSU2 PWM'
    fans = (
        AcctonAS7412_32XFan14(),
    )
    temps = (
        AcctonAS7412_32XTemp8(),
        AcctonAS7412_32XTemp9(),
        AcctonAS7412_32XTemp10(),
    )

class AcctonAS7412_32XPWMs():
    units = (
        AcctonAS7412_32XPWM1(),
        AcctonAS7412_32XPWM2(),
        AcctonAS7412_32XPWM3(),
    )

class AcctonAS7412Switch(cumulus.platform.SpectrumSwitch):
    _sfp = None
    _qsfp = AcctonAS7412QSFP28

    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = False

    # label is front panel port label.
    # adj_label is front panel port label of the adjacent port (because the
    #    adjacent label is not necessarily label+1). Note that adj_label
    #    is meaningful only when blocked_adj_split is specified.
    #
    # phy_addrs correspond to XML lane-to-module.
    # hw_ports correspond to XML local-port; the length of the tuple
    #    (when > 1) implies the max number of ports into which this port can be split.
    # blocked_adj_split Port is blocked when its adjacent port is split into
    #    this number of ports (or more)
    #
    ports = (
        _qsfp(label="1",  adj_label=None, phy_addrs=(0,0,0,0),     hw_ports=(61,62,63,64)),
        _qsfp(label="2",  adj_label="1",  phy_addrs=(1,1,1,1),     hw_ports=(63,64), blocked_adj_split=4),
        _qsfp(label="3",  adj_label=None, phy_addrs=(2,2,2,2),     hw_ports=(57,58,59,60)),
        _qsfp(label="4",  adj_label="3",  phy_addrs=(3,3,3,3),     hw_ports=(59,60), blocked_adj_split=4),
        _qsfp(label="5",  adj_label=None, phy_addrs=(4,4,4,4),     hw_ports=(53,54,55,56)),
        _qsfp(label="6",  adj_label="5",  phy_addrs=(5,5,5,5),     hw_ports=(55,56), blocked_adj_split=4),
        _qsfp(label="7",  adj_label=None, phy_addrs=(6,6,6,6),     hw_ports=(49,50,51,52)),
        _qsfp(label="8",  adj_label="7",  phy_addrs=(7,7,7,7),     hw_ports=(51,52), blocked_adj_split=4),
        _qsfp(label="9",  adj_label=None, phy_addrs=(8,8,8,8),     hw_ports=(45,46,47,48)),
        _qsfp(label="10", adj_label="9",  phy_addrs=(9,9,9,9),     hw_ports=(47,48), blocked_adj_split=4),
        _qsfp(label="11", adj_label=None, phy_addrs=(10,10,10,10), hw_ports=(41,42,43,44)),
        _qsfp(label="12", adj_label="11", phy_addrs=(11,11,11,11), hw_ports=(43,44), blocked_adj_split=4),
        _qsfp(label="13", adj_label=None, phy_addrs=(12,12,12,12), hw_ports=(37,38,39,40)),
        _qsfp(label="14", adj_label="13", phy_addrs=(13,13,13,13), hw_ports=(39,40), blocked_adj_split=4),
        _qsfp(label="15", adj_label=None, phy_addrs=(14,14,14,14), hw_ports=(33,34,35,36)),
        _qsfp(label="16", adj_label="15", phy_addrs=(15,15,15,15), hw_ports=(35,36), blocked_adj_split=4),
        _qsfp(label="17", adj_label=None, phy_addrs=(16,16,16,16), hw_ports=(1,2,3,4)),
        _qsfp(label="18", adj_label="17", phy_addrs=(17,17,17,17), hw_ports=(3,4), blocked_adj_split=4),
        _qsfp(label="19", adj_label=None, phy_addrs=(18,18,18,18), hw_ports=(5,6,7,8)),
        _qsfp(label="20", adj_label="19", phy_addrs=(19,19,19,19), hw_ports=(7,8), blocked_adj_split=4),

        _qsfp(label="21", adj_label=None, phy_addrs=(20,20,20,20), hw_ports=(9,10,11,12)),
        _qsfp(label="22", adj_label="21", phy_addrs=(21,21,21,21), hw_ports=(11,12), blocked_adj_split=4),
        _qsfp(label="23", adj_label=None, phy_addrs=(22,22,22,22), hw_ports=(13,14,15,16)),
        _qsfp(label="24", adj_label="23", phy_addrs=(23,23,23,23), hw_ports=(15,16), blocked_adj_split=4),
        _qsfp(label="25", adj_label=None, phy_addrs=(24,24,24,24), hw_ports=(17,18,19,20)),
        _qsfp(label="26", adj_label="25", phy_addrs=(25,25,25,25), hw_ports=(19,20), blocked_adj_split=4),
        _qsfp(label="27", adj_label=None, phy_addrs=(26,26,26,26), hw_ports=(21,22,23,24)),
        _qsfp(label="28", adj_label="27", phy_addrs=(27,27,27,27), hw_ports=(23,24), blocked_adj_split=4),
        _qsfp(label="29", adj_label=None, phy_addrs=(28,28,28,28), hw_ports=(25,26,27,28)),
        _qsfp(label="30", adj_label="29", phy_addrs=(29,29,29,29), hw_ports=(27,28), blocked_adj_split=4),
        _qsfp(label="31", adj_label=None, phy_addrs=(30,30,30,30), hw_ports=(29,30,31,32)),
        _qsfp(label="32", adj_label="31", phy_addrs=(31,31,31,31), hw_ports=(31,32), blocked_adj_split=4),
    )

class AcctonAS7412(cumulus.platform.Platform):
    name = 'act,as7412_32x'
    chip = cumulus.platform.SpectrumChip()
    switch = AcctonAS7412Switch(chip)
    cpld = AcctonAS7412_32XCPLD()
    sensors = AcctonAS7412_32XSensors()
    pwms = AcctonAS7412_32XPWMs()
    statusleds = None

Platform = AcctonAS7412
