#
# Copyright 2016.  Cumulus Networks, Inc.
#
import os
import subprocess
import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors
from cumulus.platforms.cel_bigstone_g_common import *

class CelBigStoneG100Switch(cumulus.platform.BCMSwitch):
    ledup = [ CelBigStoneG100LED0, CelBigStoneG100LED1 ]
    led_controllers = set([0, 1]) # 2 is unused
    _qsfp28 = CelBigStoneG100QSFP28
    _fab = CelBigStoneG100Fabric
    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = False

    ports = (

        # Tomahawk front panel ports
        _qsfp28(label="1",  serdes=( 80,  81,  82,  83), leds=(1,   0),
              tx_polarity_flip=0x0002, rx_polarity_flip=0x0000,
              tx_lane_map=0x2031, rx_lane_map=0x2031),
        _qsfp28(label="2",  serdes=( 84,  85,  86,  87), leds=(1,  12),
              tx_polarity_flip=0x0003, rx_polarity_flip=0x0000,
              tx_lane_map=0x1302, rx_lane_map=0x1302),
        _qsfp28(label="3",  serdes=( 88,  89,  90,  91), leds=(1,  24),
              tx_polarity_flip=0x0002, rx_polarity_flip=0x0000,
              tx_lane_map=0x2031, rx_lane_map=0x2031),
        _qsfp28(label="4",  serdes=( 92,  93,  94,  95), leds=(1,  36),
              tx_polarity_flip=0x000c, rx_polarity_flip=0x000f,
              tx_lane_map=0x1302, rx_lane_map=0x1302),
        _qsfp28(label="5",  serdes=( 96,  97,  98,  99), leds=(0,   0),
              tx_polarity_flip=0x000d, rx_polarity_flip=0x000f,
              tx_lane_map=0x2031, rx_lane_map=0x2031),
        _qsfp28(label="6",  serdes=(100, 101, 102, 103), leds=(0,  12),
              tx_polarity_flip=0x000c, rx_polarity_flip=0x0000,
              tx_lane_map=0x1302, rx_lane_map=0x1302),
        _qsfp28(label="7",  serdes=(104, 105, 106, 107), leds=(0,  24),
              tx_polarity_flip=0x000d, rx_polarity_flip=0x000f,
              tx_lane_map=0x2031, rx_lane_map=0x2031),
        _qsfp28(label="8",  serdes=(108, 109, 110, 111), leds=(0,  36),
              tx_polarity_flip=0x0003, rx_polarity_flip=0x0000,
              tx_lane_map=0x1302, rx_lane_map=0x1302),
        _qsfp28(label="9",  serdes=(112, 113, 114, 115), leds=(0,  48),
              tx_polarity_flip=0x000d, rx_polarity_flip=0x000f,
              tx_lane_map=0x0321, rx_lane_map=0x3012),
        _qsfp28(label="10", serdes=(116, 117, 118, 119), leds=(0,  60),
              tx_polarity_flip=0x000c, rx_polarity_flip=0x000f,
              tx_lane_map=0x1302, rx_lane_map=0x3120),
        _qsfp28(label="11", serdes=(120, 121, 122, 123), leds=(0,  72),
              tx_polarity_flip=0x0002, rx_polarity_flip=0x000f,
              tx_lane_map=0x1302, rx_lane_map=0x3120),
        _qsfp28(label="12", serdes=(124, 125, 126, 127), leds=(0,  84),
              tx_polarity_flip=0x000c, rx_polarity_flip=0x000f,
              tx_lane_map=0x2031, rx_lane_map=0x0213),
        _qsfp28(label="13", serdes=(  0,   1,   2,   3), leds=(0,  96),
              tx_polarity_flip=0x000d, rx_polarity_flip=0x000f,
              tx_lane_map=0x1302, rx_lane_map=0x3120),
        _qsfp28(label="14", serdes=(  4,   5,   6,   7), leds=(0, 108),
              tx_polarity_flip=0x000c, rx_polarity_flip=0x000f,
              tx_lane_map=0x1302, rx_lane_map=0x3120),
        _qsfp28(label="15", serdes=(  8,   9,  10,  11), leds=(0, 120),
              tx_polarity_flip=0x000d, rx_polarity_flip=0x000f,
              tx_lane_map=0x1302, rx_lane_map=0x3120),
        _qsfp28(label="16", serdes=( 12,  13,  14,  15), leds=(0, 132),
              tx_polarity_flip=0x000c, rx_polarity_flip=0x000f,
              tx_lane_map=0x1302, rx_lane_map=0x3120),

        # Line Card Tomahawk 2 backplane ports to fab #1
        _fab(label="0",  serdes=( 64,  65,  66,  67),
              tx_polarity_flip=0x0001, rx_polarity_flip=0x0000,
              tx_lane_map=0x1203, rx_lane_map=0x1023),
        _fab(label="1",  serdes=( 68,  69,  70,  71),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0000,
              tx_lane_map=0x3210, rx_lane_map=0x0123),
        _fab(label="2",  serdes=( 72,  73,  74,  75),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x000f,
              tx_lane_map=0x3210, rx_lane_map=0x0123),
        _fab(label="3",  serdes=( 76,  77,  78,  79),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0000,
              tx_lane_map=0x3102, rx_lane_map=0x2013),

        # Line Card Tomahawk 2 backplane ports to fab #2
        _fab(label="4",  serdes=( 48,  49,  50,  51),
              tx_polarity_flip=0x0001, rx_polarity_flip=0x000f,
              tx_lane_map=0x2130, rx_lane_map=0x2310),
        _fab(label="5",  serdes=( 52,  53,  54,  55),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0000,
              tx_lane_map=0x3210, rx_lane_map=0x0123),
        _fab(label="6",  serdes=( 56,  57,  58,  59),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x0000,
              tx_lane_map=0x3210, rx_lane_map=0x0123),
        _fab(label="7",  serdes=( 60,  61,  62,  63),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0000,
              tx_lane_map=0x3210, rx_lane_map=0x0213),

        # Line Card Tomahawk 2 backplane ports to fab #3
        _fab(label="8",  serdes=( 32,  33,  34,  35),
              tx_polarity_flip=0x000e, rx_polarity_flip=0x0000,
              tx_lane_map=0x1203, rx_lane_map=0x2310),
        _fab(label="9",  serdes=( 36,  37,  38,  39),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0000,
              tx_lane_map=0x3210, rx_lane_map=0x3210),
        _fab(label="10", serdes=( 40,  41,  42,  43),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x000f,
              tx_lane_map=0x0123, rx_lane_map=0x0123),
        _fab(label="11", serdes=( 44,  45,  46,  47),
              tx_polarity_flip=0x000f, rx_polarity_flip=0x000f,
              tx_lane_map=0x3210, rx_lane_map=0x3210),
        
        # Line Card Tomahawk 2 backplane ports to fab #4
        _fab(label="12", serdes=( 16,  17,  18,  19),
              tx_polarity_flip=0x000e, rx_polarity_flip=0x0000,
              tx_lane_map=0x1203, rx_lane_map=0x2310),
        _fab(label="13", serdes=( 20,  21,  22,  23),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x0000,
              tx_lane_map=0x3210, rx_lane_map=0x3210),
        _fab(label="14", serdes=( 24,  25,  26,  27),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x000f,
              tx_lane_map=0x0123, rx_lane_map=0x0123),
        _fab(label="15", serdes=( 28,  29,  30,  31),
              tx_polarity_flip=0x0000, rx_polarity_flip=0x000f,
              tx_lane_map=0x3210, rx_lane_map=0x3210),
    )


class CelBigStoneG100Lc1r(CelBigStoneG100):
    name = 'cel,bigstone_g_lc1r'
    chip = cumulus.platform.TomahawkChip()
    switch = CelBigStoneG100Switch(chip)

Platform = CelBigStoneG100Lc1r
