#
# Copyright 2013, 2014, 2015, 2016.  Cumulus Networks, Inc.
#

import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors

class CelPebblePhylessGigE(cumulus.platform.Hurricane21GPort):
    _set_led = '''
;;; CelPebblePhylessGigE

set_led:
        port a
        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        pack

        port a
        pushst PORTSTATUS_ZERO
        pack

        port a
        pushst PORTSTATUS_ZERO
        pack

        port a
        pushst PORTSTATUS_ZERO
        pack

        ret
'''

    def __init__(self, label, hw_port, prim_offset, unit, leds):
        cumulus.platform.Hurricane21GPort.__init__(self,
                                                label=label,
                                                hw_port=hw_port,
                                                phy_type=None,
                                                phy_addr=None,
                                                phy_layer='1GBaseT',
                                                prim_offset=prim_offset,
                                                leds=leds,
                                                unit=unit)

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        fn_name = '%s' % self.__class__.__name__
        call = (fn_name, self._set_led, self.hw_port_num(mode, 0))
        for led in led_index_list:
            set_calls[led] = call
            set_calls[led + 1] = None

        return set_calls


class CelPebblePhyGigE(cumulus.platform.Hurricane21GPort):

    def __init__(self, label, hw_port, phy_addr, prim_offset, unit):
        cumulus.platform.Hurricane21GPort.__init__(self,
                                                label=label,
                                                hw_port=hw_port,
                                                phy_type="54282",
                                                phy_addr=phy_addr,
                                                phy_layer='1GBaseT',
                                                prim_offset=prim_offset,
                                                leds=None,
                                                unit=unit)

class CelPebbleSFPp(cumulus.platform.Hurricane210GPort):
    _set_led = '''
;;; CelPebbleSFPp
;;; Two bits per port: yellow(active high), green(active high)
set_led:
        port a
        pushst  PORTSTATUS_ZERO
        pack
        port a
        pushst  PORTSTATUS_ZERO
        pack
        port a
        pushst  PORTSTATUS_ZERO
        pack
        port a
        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        pack

        ret
'''
    def __init__(self, label, hw_port, leds, unit):
        cumulus.platform.Hurricane210GPort.__init__(self,
                                                 label=label,
                                                 hw_port=hw_port,
                                                 phy_type=None,
                                                 phy_addr=None,
                                                 phy_layer='SFP+',
                                                 leds=leds,
                                                 unit=unit)
        self.sfp = cumulus.sfp.SFPROMDTSLabel(label, 'sfp+')

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        fn_name = '%s' % self.__class__.__name__
        call = (fn_name, self._set_led, self.led_port_num(mode, 0))
        for led in led_index_list:
            set_calls[led] = call
            set_calls[led + 1] = None

        return set_calls


class CelPebbleFabPort(cumulus.platform.Hurricane213GPort):
    is_fabric = True
    def __init__(self, label, hw_port, unit):
        cumulus.platform.Hurricane213GPort.__init__(self,
                                                label=label,
                                                hw_port=hw_port,
                                                unit=unit)

class CelPebbleCPLD(cumulus.cpld.SysFS):
    path = '/sys/bus/platform/devices/cel_pebble_cpld.0'

class CelPebbleLED(cumulus.ledup.Hurricane2LEDController):
    num_leds = 104

class CelPebbleSwitch(cumulus.platform.BCMSwitch):
    ledup = CelPebbleLED
    _phyless_gige = CelPebblePhylessGigE
    _phy_gige = CelPebblePhyGigE
    _sfp = CelPebbleSFPp
    _fab = CelPebbleFabPort
    has_phy_ucode_mdio = False # XXX unverified
    has_phy_ucode_spi_rom = False # XXX unverified

    ports = (
        _phy_gige(label="1",  hw_port='ge16',  phy_addr=0x21,  prim_offset=0x0000, unit=0),
        _phy_gige(label="2",  hw_port='ge17',  phy_addr=0x22,  prim_offset=0x0001, unit=0),
        _phy_gige(label="3",  hw_port='ge18',  phy_addr=0x23,  prim_offset=0x0002, unit=0),
        _phy_gige(label="4",  hw_port='ge19',  phy_addr=0x24,  prim_offset=0x0003, unit=0),
        _phy_gige(label="5",  hw_port='ge20',  phy_addr=0x25,  prim_offset=0x0004, unit=0),
        _phy_gige(label="6",  hw_port='ge21',  phy_addr=0x26,  prim_offset=0x0005, unit=0),
        _phy_gige(label="7",  hw_port='ge22',  phy_addr=0x27,  prim_offset=0x0006, unit=0),
        _phy_gige(label="8",  hw_port='ge23',  phy_addr=0x28,  prim_offset=0x0007, unit=0),
        _phyless_gige(label="9",  hw_port='ge0',  prim_offset=0x0800, unit=0, leds=(0,18)),
        _phyless_gige(label="10", hw_port='ge1' , prim_offset=0x0801, unit=0, leds=(0,16)),
        _phyless_gige(label="11", hw_port='ge2', prim_offset=0x0802, unit=0, leds=(0, 14)),
        _phyless_gige(label="12", hw_port='ge3', prim_offset=0x0803, unit=0, leds=(0, 12)),
        _phyless_gige(label="13", hw_port='ge4', prim_offset=0x0804, unit=0, leds=(0, 10)),
        _phyless_gige(label="14", hw_port='ge5', prim_offset=0x0805, unit=0, leds=(0, 8)),
        _phyless_gige(label="15", hw_port='ge6', prim_offset=0x0806, unit=0, leds=(0, 6)),
        _phyless_gige(label="16", hw_port='ge7', prim_offset=0x0807, unit=0, leds=(0, 4)),
        _phyless_gige(label="17", hw_port='ge8', prim_offset=0x1000, unit=0, leds=(0, 20)),
        _phyless_gige(label="18", hw_port='ge9', prim_offset=0x1001, unit=0, leds=(0, 22)),
        _phyless_gige(label="19", hw_port='ge10', prim_offset=0x1002, unit=0, leds=(0,24)),
        _phyless_gige(label="20", hw_port='ge11', prim_offset=0x1003, unit=0, leds=(0,26)),
        _phyless_gige(label="21", hw_port='ge12', prim_offset=0x1004, unit=0, leds=(0,28)),
        _phyless_gige(label="22", hw_port='ge13', prim_offset=0x1005, unit=0, leds=(0,30)),
        _phyless_gige(label="23", hw_port='ge14', prim_offset=0x1006, unit=0, leds=(0,32)),
        _phyless_gige(label="24", hw_port='ge15', prim_offset=0x1007, unit=0, leds=(0,34)),
        _phyless_gige(label="25", hw_port='ge0',  prim_offset=0x0000, unit=1, leds=(1,18)),
        _phyless_gige(label="26", hw_port='ge1',  prim_offset=0x0001, unit=1, leds=(1,16)),
        _phyless_gige(label="27", hw_port='ge2',  prim_offset=0x0002, unit=1, leds=(1,14)),
        _phyless_gige(label="28", hw_port='ge3',  prim_offset=0x0003, unit=1, leds=(1,12)),
        _phyless_gige(label="29", hw_port='ge4',  prim_offset=0x0004, unit=1, leds=(1,10)),
        _phyless_gige(label="30", hw_port='ge5',  prim_offset=0x0005, unit=1, leds=(1,8)),
        _phyless_gige(label="31", hw_port='ge6',  prim_offset=0x0006, unit=1, leds=(1,6)),
        _phyless_gige(label="32", hw_port='ge7',  prim_offset=0x0007, unit=1, leds=(1,4)),
        _phyless_gige(label="33", hw_port='ge8',  prim_offset=0x0800, unit=1, leds=(1,20)),
        _phyless_gige(label="34", hw_port='ge9',  prim_offset=0x0801, unit=1, leds=(1,22)),
        _phyless_gige(label="35", hw_port='ge10', prim_offset=0x0802, unit=1, leds=(1,24)),
        _phyless_gige(label="36", hw_port='ge11', prim_offset=0x0803, unit=1, leds=(1,26)),
        _phyless_gige(label="37", hw_port='ge12', prim_offset=0x0804, unit=1, leds=(1,28)),
        _phyless_gige(label="38", hw_port='ge13', prim_offset=0x0805, unit=1, leds=(1,30)),
        _phyless_gige(label="39", hw_port='ge14', prim_offset=0x0806, unit=1, leds=(1,32)),
        _phyless_gige(label="40", hw_port='ge15', prim_offset=0x0807, unit=1, leds=(1,34)),
        _phy_gige(label="41", hw_port='ge16', phy_addr=0x2a, prim_offset=0x1000, unit=1),
        _phy_gige(label="42", hw_port='ge17', phy_addr=0x2b, prim_offset=0x1001, unit=1),
        _phy_gige(label="43", hw_port='ge18', phy_addr=0x2c, prim_offset=0x1002, unit=1),
        _phy_gige(label="44", hw_port='ge19', phy_addr=0x2d, prim_offset=0x1003, unit=1),
        _phy_gige(label="45", hw_port='ge20', phy_addr=0x2e, prim_offset=0x1004, unit=1),
        _phy_gige(label="46", hw_port='ge21', phy_addr=0x2f, prim_offset=0x1005, unit=1),
        _phy_gige(label="47", hw_port='ge22', phy_addr=0x30, prim_offset=0x1006, unit=1),
        _phy_gige(label="48", hw_port='ge23', phy_addr=0x31, prim_offset=0x1007, unit=1),

        _sfp(label="49", hw_port='xe0', leds=(0, 2), unit=0),
        _sfp(label="50", hw_port='xe1', leds=(0, 0), unit=0),
        _sfp(label="51", hw_port='xe1', leds=(1, 2), unit=1),
        _sfp(label="52", hw_port='xe0', leds=(1, 0), unit=1),

        _fab(label="0", hw_port='hg0', unit=0),
        _fab(label="1", hw_port='hg1', unit=0),
        _fab(label="2", hw_port='hg0', unit=1),
        _fab(label="3", hw_port='hg1', unit=1),
    )

CelPebble_i2c_path = '/sys/bus/i2c/devices/'

''' lm75 temp sensor 1 '''
CelPebble_i2c_0_4a_path = CelPebble_i2c_path + '0-004a'

''' lm75 temp sensor 2 '''
CelPebble_i2c_16_48_path = CelPebble_i2c_path + '16-0048'

''' lm75 temp sensor 2 '''
CelPebble_i2c_16_49_path = CelPebble_i2c_path + '16-0049'

class CelPebbleTemp1(cumulus.sensors.Temp_Unit):
    name = 'Temp1'
    description = cumulus.sensors.Temp_Unit.mac_board_sensor_name
    description = 'Ambient Temp Sensor on CPU Board'
    driver_path = CelPebble_i2c_0_4a_path
    driver_hwmon = ['temp1']
    temp_fan_min = 25
    temp_max = 60
    temp_crit = 70

class CelPebbleTemp2(cumulus.sensors.Temp_Unit):
    name = 'Temp2'
    description = 'Inlet Ambient Temp Sensor'
    driver_path = CelPebble_i2c_16_48_path
    driver_hwmon = ['temp1']
    temp_fan_min = 35
    temp_max = 90
    temp_crit = 95

class CelPebbleTemp3(cumulus.sensors.Temp_Unit):
    name = 'Temp3'
    description = 'Rear Outlet Temp Sensor'
    driver_path = CelPebble_i2c_16_49_path
    driver_hwmon = ['temp1']
    temp_fan_min = 40
    temp_max = 90
    temp_crit = 95

CelPebble_core_path = '/sys/devices/platform/coretemp.0'
class CelPebbleTemp4(cumulus.sensors.Temp_Unit):
    name = 'Temp4'
    description = 'Intel CPU die sensor'
    driver_path = CelPebble_core_path
    driver_hwmon = ['temp2']
    temp_fan_min = 50
    temp_max = 90
    temp_crit = 95

class CelPebbleTemp5(cumulus.sensors.Temp_Unit):
    name = 'Temp5'
    description = 'Intel CPU die sensor'
    driver_path = CelPebble_core_path
    driver_hwmon = ['temp4']
    temp_fan_min = 50
    temp_max = 90
    temp_crit = 95


class CelPebbleFan1(cumulus.sensors.Fan_Unit):
    name = 'Fan01'
    driver_path =  CelPebbleCPLD.path

    # driver is fan speed
    driver_hwmon = ['fan1']
    # status is "cpld_hwmon"
    cpld_hwmon = ['fan1']
    # pwm is to manage Fans
    pwm_hwmon = ['pwm1']

    fan_min = 2500
    fan_max = 20000
    # According to Pebble Fan Speed Control Strategy v0.004,
    # default fan pwm should be 35%
    minpwm= 100

class CelPebblePWM1(cumulus.sensors.PWM_Unit):
    fans = (CelPebbleFan1(),
            )
    temps = (CelPebbleTemp1(),
             CelPebbleTemp2(),
             CelPebbleTemp3(),
             CelPebbleTemp4(),
             CelPebbleTemp5(),
             )

class CelPebbleSensors():
    units = ( CelPebbleFan1(),
              CelPebbleTemp1(),
              CelPebbleTemp2(),
              CelPebbleTemp3(),
              CelPebbleTemp4(),
              CelPebbleTemp5(),
              )

class CelPebblePWMs():
    units = (CelPebblePWM1(),
             )

class CelPebbleSystemLED(cumulus.sensors.StatusLED_Unit):
    name = 'System'
    led_location = 'led_system_status'
    bad_led_color = 'amber_blinking'
    led_path = CelPebbleCPLD.path

class CelPebbleLEDs():
    units = (CelPebbleSystemLED(),
             )

class CelPebble(cumulus.platform.Platform):
    name = 'cel,pebble'
    chip = cumulus.platform.Hurricane2Chip()
    switch = CelPebbleSwitch(chip)
    cpld = CelPebbleCPLD()
    statusleds = CelPebbleLEDs()
    pwms = CelPebblePWMs()
    sensors = CelPebbleSensors()

Platform = CelPebble
