#
# Copyright 2013,2014,2015,2016,2017.  Cumulus Networks,Inc. All rights reserved
#
# This software is subject to the Cumulus Networks End User License
# Agreement available  at the following locations:
#
# Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
# Cumulus Linux systems: /usr/share/cumulus/EULA.txt

import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors

vhwmon_path = '/sys/class/misc/vhwmon/bmc:'

class CelPebblebPhylessGigE(cumulus.platform.Hurricane21GPort):
    _set_led = '''
;;; CelPebblebPhylessGigE

set_led:
        port a
        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        pack

        port a
        pushst PORTSTATUS_ZERO
        pack

        port a
        pushst PORTSTATUS_ZERO
        pack

        port a
        pushst PORTSTATUS_ZERO
        pack

        ret
'''

    def __init__(self, label, hw_port, prim_offset, unit, leds):
        cumulus.platform.Hurricane21GPort.__init__(self,
                                                label=label,
                                                hw_port=hw_port,
                                                phy_type=None,
                                                phy_addr=None,
                                                phy_layer='1GBaseT',
                                                prim_offset=prim_offset,
                                                leds=leds,
                                                unit=unit)

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        fn_name = '%s' % self.__class__.__name__
        call = (fn_name, self._set_led, self.hw_port_num(mode, 0))
        for led in led_index_list:
            set_calls[led] = call
            set_calls[led + 1] = None

        return set_calls


class CelPebblebPhyGigE(cumulus.platform.Hurricane21GPort):

    def __init__(self, label, hw_port, phy_addr, prim_offset, unit):
        cumulus.platform.Hurricane21GPort.__init__(self,
                                                label=label,
                                                hw_port=hw_port,
                                                phy_type="54282",
                                                phy_addr=phy_addr,
                                                phy_layer='1GBaseT',
                                                prim_offset=prim_offset,
                                                leds=None,
                                                unit=unit)

class CelPebblebSFPp(cumulus.platform.Hurricane210GPort):
    _set_led = '''
;;; CelPebblebSFPp
;;; Two bits per port: yellow(active high), green(active high)
set_led:
        port a
        pushst  PORTSTATUS_ZERO
        pack
        port a
        pushst  PORTSTATUS_ZERO
        pack
        port a
        pushst  PORTSTATUS_ZERO
        pack
        port a
        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        pack

        ret
'''
    def __init__(self, label, hw_port, leds, unit):
        cumulus.platform.Hurricane210GPort.__init__(self,
                                                 label=label,
                                                 hw_port=hw_port,
                                                 phy_type=None,
                                                 phy_addr=None,
                                                 phy_layer='SFP+',
                                                 leds=leds,
                                                 unit=unit)
        self.sfp = cumulus.sfp.SFPROMDTSLabel(label, 'sfp+')

    def set_leds(self, mode, led_port, sub, led_index_list):
        self._check_mode(mode)

        set_calls = {}

        fn_name = '%s' % self.__class__.__name__
        call = (fn_name, self._set_led, self.led_port_num(mode, 0))
        for led in led_index_list:
            set_calls[led] = call
            set_calls[led + 1] = None

        return set_calls


class CelPebblebFabPort(cumulus.platform.Hurricane213GPort):
    is_fabric = True
    def __init__(self, label, hw_port, unit):
        cumulus.platform.Hurricane213GPort.__init__(self,
                                                label=label,
                                                hw_port=hw_port,
                                                unit=unit)

class CelPebblebCPLD(cumulus.cpld.SysFS):
    path = '/sys/bus/platform/devices/cel_pebble_b_cpld.0'

class CelPebblebLED(cumulus.ledup.Hurricane2LEDController):
    num_leds = 104

class CelPebblebSwitch(cumulus.platform.BCMSwitch):
    ledup = CelPebblebLED
    _phyless_gige = CelPebblebPhylessGigE
    _phy_gige = CelPebblebPhyGigE
    _sfp = CelPebblebSFPp
    _fab = CelPebblebFabPort
    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = False

# pebble_b has different rj45 wiring than pebble (ports to switches)
# pebble   has 0-24 from bcm56150_1,      25-48 from bcm565150_2
# pebble_b has 9-16,33-48 from bcm56150_1 (unit 0),
#              1-8, 17-32 from bcm56150_2 (unit 1)
# Note that ports 7-12, 19-24, 31-36 and 43-48 swap odd and even.
# Determined experimentally by walking through the ports
# Swapped wholesale (that is, swapped the label lines, then re-sorted).
# Port LEDs still not working for 1-16 (external phy), but ports work
    ports = (
        _phy_gige(label="1",  hw_port='ge16', phy_addr=0x21, prim_offset=0x0000, unit=1),
        _phy_gige(label="2",  hw_port='ge17', phy_addr=0x22, prim_offset=0x0001, unit=1),
        _phy_gige(label="3",  hw_port='ge18', phy_addr=0x23, prim_offset=0x0002, unit=1),
        _phy_gige(label="4",  hw_port='ge19', phy_addr=0x24, prim_offset=0x0003, unit=1),
        _phy_gige(label="5",  hw_port='ge20', phy_addr=0x25, prim_offset=0x0004, unit=1),
        _phy_gige(label="6",  hw_port='ge21', phy_addr=0x26, prim_offset=0x0005, unit=1),
        _phy_gige(label="7",  hw_port='ge23', phy_addr=0x28, prim_offset=0x0007, unit=1),
        _phy_gige(label="8",  hw_port='ge22', phy_addr=0x27, prim_offset=0x0006, unit=1),
        _phy_gige(label="9",  hw_port='ge17', phy_addr=0x2b, prim_offset=0x1001, unit=0),
        _phy_gige(label="10", hw_port='ge16', phy_addr=0x2a, prim_offset=0x1000, unit=0),
        _phy_gige(label="11", hw_port='ge19', phy_addr=0x2d, prim_offset=0x1003, unit=0),
        _phy_gige(label="12", hw_port='ge18', phy_addr=0x2c, prim_offset=0x1002, unit=0),
        _phy_gige(label="13", hw_port='ge20', phy_addr=0x2e, prim_offset=0x1004, unit=0),
        _phy_gige(label="14", hw_port='ge21', phy_addr=0x2f, prim_offset=0x1005, unit=0),
        _phy_gige(label="15", hw_port='ge22', phy_addr=0x30, prim_offset=0x1006, unit=0),
        _phy_gige(label="16", hw_port='ge23', phy_addr=0x31, prim_offset=0x1007, unit=0),
        _phyless_gige(label="17", hw_port='ge0',  prim_offset=0x0000, unit=1, leds=(1,18)),
        _phyless_gige(label="18", hw_port='ge1',  prim_offset=0x0001, unit=1, leds=(1,16)),
        _phyless_gige(label="19", hw_port='ge3',  prim_offset=0x0003, unit=1, leds=(1,14)),
        _phyless_gige(label="20", hw_port='ge2',  prim_offset=0x0002, unit=1, leds=(1,12)),
        _phyless_gige(label="21", hw_port='ge5',  prim_offset=0x0005, unit=1, leds=(1,10)),
        _phyless_gige(label="22", hw_port='ge4',  prim_offset=0x0004, unit=1, leds=(1,8)),
        _phyless_gige(label="23", hw_port='ge7',  prim_offset=0x0007, unit=1, leds=(1,6)),
        _phyless_gige(label="24", hw_port='ge6',  prim_offset=0x0006, unit=1, leds=(1,4)),
        _phyless_gige(label="25", hw_port='ge8',  prim_offset=0x0800, unit=1, leds=(1,20)),
        _phyless_gige(label="26", hw_port='ge9',  prim_offset=0x0801, unit=1, leds=(1,22)),
        _phyless_gige(label="27", hw_port='ge10', prim_offset=0x0802, unit=1, leds=(1,24)),
        _phyless_gige(label="28", hw_port='ge11', prim_offset=0x0803, unit=1, leds=(1,26)),
        _phyless_gige(label="29", hw_port='ge12', prim_offset=0x0804, unit=1, leds=(1,28)),
        _phyless_gige(label="30", hw_port='ge13', prim_offset=0x0805, unit=1, leds=(1,30)),
        _phyless_gige(label="31", hw_port='ge15', prim_offset=0x0807, unit=1, leds=(1,32)),
        _phyless_gige(label="32", hw_port='ge14', prim_offset=0x0806, unit=1, leds=(1,34)),
        _phyless_gige(label="33", hw_port='ge1',  prim_offset=0x0801, unit=0, leds=(0,18)),
        _phyless_gige(label="34", hw_port='ge0' , prim_offset=0x0800, unit=0, leds=(0,16)),
        _phyless_gige(label="35", hw_port='ge3',  prim_offset=0x0803, unit=0, leds=(0, 14)),
        _phyless_gige(label="36", hw_port='ge2',  prim_offset=0x0802, unit=0, leds=(0, 12)),
        _phyless_gige(label="37", hw_port='ge4',  prim_offset=0x0804, unit=0, leds=(0, 10)),
        _phyless_gige(label="38", hw_port='ge5',  prim_offset=0x0805, unit=0, leds=(0, 8)),
        _phyless_gige(label="39", hw_port='ge6',  prim_offset=0x0806, unit=0, leds=(0, 6)),
        _phyless_gige(label="40", hw_port='ge7',  prim_offset=0x0807, unit=0, leds=(0, 4)),
        _phyless_gige(label="41", hw_port='ge8',  prim_offset=0x1000, unit=0, leds=(0, 20)),
        _phyless_gige(label="42", hw_port='ge9',  prim_offset=0x1001, unit=0, leds=(0, 22)),
        _phyless_gige(label="43", hw_port='ge11', prim_offset=0x1003, unit=0, leds=(0,24)),
        _phyless_gige(label="44", hw_port='ge10', prim_offset=0x1002, unit=0, leds=(0,26)),
        _phyless_gige(label="45", hw_port='ge13', prim_offset=0x1005, unit=0, leds=(0,28)),
        _phyless_gige(label="46", hw_port='ge12', prim_offset=0x1004, unit=0, leds=(0,30)),
        _phyless_gige(label="47", hw_port='ge15', prim_offset=0x1007, unit=0, leds=(0,32)),
        _phyless_gige(label="48", hw_port='ge14', prim_offset=0x1006, unit=0, leds=(0,34)),

        _sfp(label="49", hw_port='xe0', leds=(0, 2), unit=0),
        _sfp(label="50", hw_port='xe1', leds=(0, 0), unit=0),
        _sfp(label="51", hw_port='xe1', leds=(1, 2), unit=1),
        _sfp(label="52", hw_port='xe0', leds=(1, 0), unit=1),

        _fab(label="0", hw_port='hg0', unit=0),
        _fab(label="1", hw_port='hg1', unit=0),
        _fab(label="2", hw_port='hg0', unit=1),
        _fab(label="3", hw_port='hg1', unit=1),
    )

CelPebbleb_i2c_path = '/sys/bus/i2c/devices/'

''' lm75 temp sensor 1 '''
CelPebbleb_i2c_0_4a_path = CelPebbleb_i2c_path + '13-004a'

class CelPebblebTemp1(cumulus.sensors.Temp_Unit):
    name = 'Temp1'
    description = cumulus.sensors.Temp_Unit.mac_board_sensor_name
    description = 'CPU Board Ambient'
    driver_path = CelPebbleb_i2c_0_4a_path
    driver_hwmon = ['temp1']
    temp_fan_min = 60
    temp_max = 60
    temp_crit = 65

CelPebbleb_core_path = '/sys/devices/platform/coretemp.0'
class CelPebblebTemp2(cumulus.sensors.Temp_Unit):
    name = 'Temp2'
    description = 'Intel CPU core 0 die sensor'
    driver_path = CelPebbleb_core_path
    driver_hwmon = ['temp2']
    temp_fan_min = 50
    temp_max = 110
    temp_crit = 110

class CelPebblebTemp4(cumulus.sensors.Temp_Unit):
    name = 'Temp3'
    description = 'Intel CPU core 2 die sensor'
    driver_path = CelPebbleb_core_path
    driver_hwmon = ['temp4']
    temp_fan_min = 50
    temp_max = 110
    temp_crit = 110

class CelPebblebSysTemp(cumulus.sensors.Temp_Unit):
    driver_path = vhwmon_path + 'temp'

class CelPebblebTemp3(CelPebblebSysTemp):
    name = 'Inlet_Temp'
    description = 'Inlet Ambient Temp Sensor'
    temp_crit = 56
    temp_max = 50
    driver_hwmon = ['temp1']

class CelPebblebTemp5(CelPebblebSysTemp):
    name = 'Outlet_Temp'
    description = 'Rear Outlet Temp Sensor'
    temp_crit = 71
    temp_max = 65
    driver_hwmon = ['temp2']

class CelPebblebSysFan(cumulus.sensors.Fan_Unit):
    driver_path = vhwmon_path + 'fan'
    no_pwm = True

class CelPebblebFan1(CelPebblebSysFan):
    name = 'Fan1'
    description = 'Fan01'
    driver_hwmon = ['fan1']

class CelPebblebFan2(CelPebblebSysFan):
    name = 'Fan2'
    description = 'Fan02'
    driver_hwmon = ['fan2']

class CelPebblebSensors():
    units = ( CelPebblebFan1(),
              CelPebblebFan2(),
              CelPebblebTemp1(),
              CelPebblebTemp2(),
              CelPebblebTemp3(),
              CelPebblebTemp4(),
              CelPebblebTemp5(),
              )

class CelPebblebBMC(cumulus.platform.BMC):
    name = 'bmc'
    type = 'ipmibmc'
    ignores = { 'CPU_U35' }
    renames = { 'Switch_U13': 'Inlet_Temp', 'Switch_U7': 'Outlet_Temp' }

class CelPebblebSystemLED(cumulus.sensors.StatusLED_Unit):
    name = 'System'
    led_location = 'led_system_status'
    bad_led_color = 'green_blinking'
    led_path = CelPebblebCPLD.path

class CelPebblebLEDs():
    units = (CelPebblebSystemLED(),
             )

class CelPebbleb(cumulus.platform.Platform):
    name = 'cel,pebble_b'
    chip = cumulus.platform.Hurricane2Chip()
    switch = CelPebblebSwitch(chip)
    cpld = CelPebblebCPLD()
    statusleds = CelPebblebLEDs()
    sensors = CelPebblebSensors()
    bmc = CelPebblebBMC()
    pwms = None

Platform = CelPebbleb
