#
# Copyright 2016.  Cumulus Networks, Inc.
#

import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors

class DNI3448PSFPp(cumulus.platform.Helix410GPort):
    _set_led = '''
;;; DNI3448PSFPp
;;;
;;; 3 bits  per port, 1: off, 0: on
;;;
;;; 1st bit: activity
;;; 2nd bit: green
;;; 3rd bit: yellow
;;;
set_led:
        port a
        pushst  PORTSTATUS_ONE
	pack

        port a
        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        tinv
        pack

        port a
        pushst PORTSTATUS_ONE
        pack

        ret

'''
    def __init__(self, label, hw_port, leds, phy_addr):
        cumulus.platform.Helix410GPort.__init__(self,
                                                 label=label,
                                                 hw_port=hw_port,
                                                 phy_type='84752',
                                                 phy_addr=phy_addr,
                                                 phy_layer='SFP+',
                                                 leds=leds,
                                                 unit=0)
        self.sfp = cumulus.sfp.SFPROMBCM(phy_addr, 'sfp+')

    def set_leds(self, mode, led_port, sub, led_index_list):
        '''
        self._check_mode(mode)
        '''

        set_calls = {}

        fn_name = '%s' % self.__class__.__name__
        led_port = self.led_port_num(mode, 0)

        call = (fn_name, self._set_led, led_port)

        led_index = led_index_list[0]
        set_calls[led_index] = call
        set_calls[led_index + 1] = None
        set_calls[led_index + 2] = None

        return set_calls

class DNI3448PGigE(cumulus.platform.Helix41GPort):
    _set_led = '''
;;; DNI3448P1GigE
;;;
;;; 4 bits  per port, 1: off, 0: on
;;;
;;; 1st bit: activity
;;; 2nd bit: POE
;;; 3nd bit: green
;;; 4rd bit: yellow
;;;
set_led:
	port a
        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        tinv
        pack

        port a
        pushst PORTSTATUS_ONE
        pack

	port a
        pushst  PORTSTATUS_RX
        pushst  PORTSTATUS_TX
        tor
        push    (BLINK_STATE_P)
        tand
        tinv                    ; flash off with activity
        pushst  PORTSTATUS_LINKEN
        tand
        tinv
        pack

        port a
        pushst PORTSTATUS_ONE
        pack

        ret

'''
    def __init__(self, label, hw_port, phy_addr, prim_offset, leds):
        cumulus.platform.Helix41GPort.__init__(self,
                                                label=label,
                                                hw_port=hw_port,
                                                phy_type="54282",
                                                phy_addr=phy_addr,
                                                phy_layer='1GBaseT',
                                                prim_offset=prim_offset,
                                                leds=leds,
                                                unit=0)

    def set_leds(self, mode, led_port, sub, led_index_list):
        '''
        self._check_mode(mode)
        '''

        set_calls = {}

        fn_name = '%s' % self.__class__.__name__
        led_port = self.led_port_num(mode, 0)

        call = (fn_name, self._set_led, led_port)

        led_index = led_index_list[0]
        set_calls[led_index] = call
        set_calls[led_index + 1] = None
        set_calls[led_index + 2] = None
        set_calls[led_index + 3] = None

        return set_calls

class DNI3448PLED(cumulus.ledup.Helix4LEDController):
    num_leds = 255

class DNI3448PSwitch(cumulus.platform.BCMSwitch):
    ledup = DNI3448PLED
    _gige = DNI3448PGigE
    _sfp = DNI3448PSFPp
    has_phy_ucode_mdio = False
    has_phy_ucode_spi_rom = True

    ports = (
        _gige(label="1",   hw_port='ge0',   phy_addr=0x1, prim_offset=0x0100, leds=None),
        _gige(label="2",   hw_port='ge1',   phy_addr=0x2, prim_offset=0x0101, leds=None),
        _gige(label="3",   hw_port='ge2',   phy_addr=0x3, prim_offset=0x0102, leds=None),
        _gige(label="4",   hw_port='ge3',   phy_addr=0x4, prim_offset=0x0103, leds=None),
        _gige(label="5",   hw_port='ge4',   phy_addr=0x5, prim_offset=0x0104, leds=None),
        _gige(label="6",   hw_port='ge5',   phy_addr=0x6, prim_offset=0x0105, leds=None),
        _gige(label="7",   hw_port='ge6',   phy_addr=0x7, prim_offset=0x0106, leds=None),
        _gige(label="8",   hw_port='ge7',   phy_addr=0x8, prim_offset=0x0107, leds=None),

        _gige(label="9",   hw_port='ge8',   phy_addr=0xa, prim_offset=0x0900, leds=None),
        _gige(label="10",  hw_port='ge9',   phy_addr=0xb, prim_offset=0x0901, leds=None),
        _gige(label="11",  hw_port='ge10',  phy_addr=0xc, prim_offset=0x0902, leds=None),
        _gige(label="12",  hw_port='ge11',  phy_addr=0xd, prim_offset=0x0903, leds=None),
        _gige(label="13",  hw_port='ge12',  phy_addr=0xe, prim_offset=0x0904, leds=None),
        _gige(label="14",  hw_port='ge13',  phy_addr=0xf, prim_offset=0x0905, leds=None),
        _gige(label="15",  hw_port='ge14',  phy_addr=0x10, prim_offset=0x0906, leds=None),
        _gige(label="16",  hw_port='ge15',  phy_addr=0x11, prim_offset=0x0907, leds=None),

        _gige(label="17",  hw_port='ge16',  phy_addr=0x13, prim_offset=0x1100, leds=None),
        _gige(label="18",  hw_port='ge17',  phy_addr=0x14, prim_offset=0x1101, leds=None),
        _gige(label="19",  hw_port='ge18',  phy_addr=0x15, prim_offset=0x1102, leds=None),
        _gige(label="20",  hw_port='ge19',  phy_addr=0x16, prim_offset=0x1103, leds=None),
        _gige(label="21",  hw_port='ge20',  phy_addr=0x17, prim_offset=0x1104, leds=None),
        _gige(label="22",  hw_port='ge21',  phy_addr=0x18, prim_offset=0x1105, leds=None),
        _gige(label="23",  hw_port='ge22',  phy_addr=0x19, prim_offset=0x1106, leds=None),
        _gige(label="24",  hw_port='ge23',  phy_addr=0x1a, prim_offset=0x1107, leds=None),

        _gige(label="25",  hw_port='ge24',  phy_addr=0x21, prim_offset=0x1900, leds=None),
        _gige(label="26",  hw_port='ge25',  phy_addr=0x22, prim_offset=0x1901, leds=None),
        _gige(label="27",  hw_port='ge26',  phy_addr=0x23, prim_offset=0x1902, leds=None),
        _gige(label="28",  hw_port='ge27',  phy_addr=0x24, prim_offset=0x1903, leds=None),
        _gige(label="29",  hw_port='ge28',  phy_addr=0x25, prim_offset=0x1904, leds=None),
        _gige(label="30",  hw_port='ge29',  phy_addr=0x26, prim_offset=0x1905, leds=None),
        _gige(label="31",  hw_port='ge30',  phy_addr=0x27, prim_offset=0x1906, leds=None),
        _gige(label="32",  hw_port='ge31',  phy_addr=0x28, prim_offset=0x1907, leds=None),

        _gige(label="33",  hw_port='ge32',  phy_addr=0x2a, prim_offset=0x2100, leds=None),
        _gige(label="34",  hw_port='ge33',  phy_addr=0x2b, prim_offset=0x2101, leds=None),
        _gige(label="35",  hw_port='ge34',  phy_addr=0x2c, prim_offset=0x2102, leds=None),
        _gige(label="36",  hw_port='ge35',  phy_addr=0x2d, prim_offset=0x2103, leds=None),
        _gige(label="37",  hw_port='ge36',  phy_addr=0x2e, prim_offset=0x2104, leds=None),
        _gige(label="38",  hw_port='ge37',  phy_addr=0x2f, prim_offset=0x2105, leds=None),
        _gige(label="39",  hw_port='ge38',  phy_addr=0x30, prim_offset=0x2106, leds=None),
        _gige(label="40",  hw_port='ge39',  phy_addr=0x31, prim_offset=0x2107, leds=None),

        _gige(label="41",  hw_port='ge40',  phy_addr=0x33, prim_offset=0x2900, leds=None),
        _gige(label="42",  hw_port='ge41',  phy_addr=0x34, prim_offset=0x2901, leds=None),
        _gige(label="43",  hw_port='ge42',  phy_addr=0x35, prim_offset=0x2902, leds=None),
        _gige(label="44",  hw_port='ge43',  phy_addr=0x36, prim_offset=0x2903, leds=None),
        _gige(label="45",  hw_port='ge44',  phy_addr=0x37, prim_offset=0x2904, leds=None),
        _gige(label="46",  hw_port='ge45',  phy_addr=0x38, prim_offset=0x2905, leds=None),
        # Ports 47 and 48 can be used as 1GT POE or 10G SFP+. We will be using
        # these 2 ports as 1GT POE.
        _gige(label="47",  hw_port='ge46',  phy_addr=0x39, prim_offset=0x2906, leds=(0,249)),
        _gige(label="48",  hw_port='ge47',  phy_addr=0x3a, prim_offset=0x2907, leds=(0,245)),

        _sfp(label="49",   hw_port="hg0",   phy_addr=0x48, leds=(0,241)),
        _sfp(label="50",   hw_port="hg1",   phy_addr=0x49, leds=(0,238)),
    )


class DNI3448PCPLD(cumulus.cpld.SysFS):
    path = '/sys/bus/i2c/devices/0-0028'
    types = { 'port_raw_ctrl' : 'L' }

class DNI3448PPSU1(cumulus.sensors.PSU_Unit):
    name = 'PSU1'
    cpld_hwmon = ['psu_pwr1']

class DNI3448PPSU2(cumulus.sensors.PSU_Unit):
    name = 'PSU2'
    cpld_hwmon = ['psu_pwr2']

class DNI3448PTemp1(cumulus.sensors.Temp_Unit):
    name = 'Temp1'
    description = "Close to MAC sensor"
    driver_path = '/sys/bus/i2c/devices/0-0049'
    driver_hwmon = ['temp1']
    temp_max_hyst = 50
    temp_max = 75
    temp_crit = 105

class DNI3448PTemp2(cumulus.sensors.Temp_Unit):
    name = 'Temp2'
    description = "Close to PHY sensor"
    driver_path = '/sys/bus/i2c/devices/0-004a'
    driver_hwmon = ['temp1']
    temp_max_hyst = 50
    temp_max = 75
    temp_crit = 100

class DNI3448PFan1(cumulus.sensors.Fan_Unit):
    name = 'Fan1'
    driver_path = '/sys/bus/i2c/devices/1-002c'
    cpld_hwmon = ['fan_tray']
    driver_hwmon = ['fan1']
    rpmmode = 1
    fan_min = 7000
    fan_max = 12000

class DNI3448PFan2(cumulus.sensors.Fan_Unit):
    name = 'Fan2'
    driver_path = '/sys/bus/i2c/devices/1-002c'
    cpld_hwmon = ['fan_tray']
    driver_hwmon = ['fan2']
    rpmmode = 1
    fan_min = 7000
    fan_max = 12000

class DNI3448PSystemLED(cumulus.sensors.StatusLED_Unit):
    name = 'System'
    led_location = 'led_system'
    bad_led_color = 'red_blinking'

class DNI3448PFanLED(cumulus.sensors.StatusLED_Unit):
    name = 'Fan'
    sources = ['Fan1', 'Fan2']
    led_location = 'led_fan'
    bad_led_color = 'red'

class DNI3448PPSU1LED(cumulus.sensors.StatusLED_Unit):
    name = 'PSU1'
    sources = ['PSU1']
    led_location = 'led_psu1'
    bad_led_color = 'green_blinking'

class DNI3448PPSU2LED(cumulus.sensors.StatusLED_Unit):
    name = 'PSU2'
    sources = ['PSU2']
    led_location = 'led_psu2'
    bad_led_color = 'green_blinking'

class DNI3448PMasterLED(cumulus.sensors.StatusLED_Unit):
    name = 'Master'
    led_location = 'led_master'
    bad_led_color = 'off'
    good_led_color = 'off'

class DNI3448PTempLED(cumulus.sensors.StatusLED_Unit):
    name = 'Temp'
    sources = ['Temp1', 'Temp2']
    led_location = 'led_temp'
    bad_led_color = 'red'

class DNI3448PPWM1(cumulus.sensors.PWM_Unit):
    fans = (DNI3448PFan1(),
            DNI3448PFan2(),
            )
    temps = (DNI3448PTemp1(),
             DNI3448PTemp2(),
             )

class DNI3448PSensors():
    units = ( DNI3448PPSU1(),
              DNI3448PPSU2(),
              DNI3448PFan1(),
              DNI3448PFan2(),
              DNI3448PTemp1(),
              DNI3448PTemp2(),
              )

class DNI3448PPWMs():
    units = (DNI3448PPWM1(),
             )

class DNI3448PLEDs():
    units = (DNI3448PSystemLED(),
             DNI3448PFanLED(),
             DNI3448PPSU1LED(),
             DNI3448PPSU2LED(),
             DNI3448PTempLED(),
             DNI3448PMasterLED(),
             )

class DNI3448P(cumulus.platform.Platform):
    name = 'dni,3448p'
    cpld = DNI3448PCPLD()
    chip = cumulus.platform.Helix4Chip()
    switch = DNI3448PSwitch(chip)
    pwms = DNI3448PPWMs()
    sensors = DNI3448PSensors()
    statusleds = DNI3448PLEDs()

Platform = DNI3448P
