#
# Copyright 2016.  Cumulus Networks, Inc.
#

import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors
import cumulus.platforms.quanta_ly6

class QuantaLY6_PSU(cumulus.sensors.PSU_Unit):
    def find_and_add_attrs(self):
        err = False
        if self.get_gpio_value(self.psu_present) == '0':
            self.attrs['psu_pwr1'] = 'installed'
        else:
            self.attrs['psu_pwr1'] = 'not_installed'
            err = True
        if self.get_gpio_value(self.psu_dc_ok) == '1':
            self.attrs['psu_pwr1'] += ', dc_ok'
        else:
            self.attrs['psu_pwr1'] += ', dc_not_ok'
            err = True
        if self.get_gpio_value(self.psu_ac_ok) == '1':
            self.attrs['psu_pwr1'] += ', ac_ok'
        else:
            self.attrs['psu_pwr1'] += ', ac_not_ok'
            err = True

        if err is False:
            self.attrs['psu_pwr1'] += ', ok'

        return 0

class QuantaLY6_Fan(cumulus.sensors.Fan_Unit):
    fan_min = 5000
    fan_max = 16500
    rpmmode = 1
    driver_path = '/sys/devices/soc.0/ffe03000.i2c/i2c-0/0-004e'

    def find_and_add_attrs(self):
        if self.get_gpio_value(self.fan_present) == '0':
            self.attrs['fan_0'] = 'installed, ok'
        else:
            self.attrs['fan_0'] = 'not_installed'
        return cumulus.sensors.Fan_Unit.find_and_add_attrs(self)

class QuantaLY6Fan1(QuantaLY6_Fan):
    fan_present = 'gpio172'
    name = 'Fan1'
    driver_hwmon = ['fan1']

class QuantaLY6Fan2(QuantaLY6_Fan):
    fan_present = 'gpio173'
    name = 'Fan2'
    driver_hwmon = ['fan2']

class QuantaLY6Fan3(QuantaLY6_Fan):
    fan_present = 'gpio174'
    name = 'Fan3'
    driver_hwmon = ['fan3']

class QuantaLY6Fan4(QuantaLY6_Fan):
    fan_present = 'gpio172'
    name = 'Fan4'
    driver_hwmon = ['fan5']

class QuantaLY6Fan5(QuantaLY6_Fan):
    fan_present = 'gpio173'
    name = 'Fan5'
    driver_hwmon = ['fan6']

class QuantaLY6Fan6(QuantaLY6_Fan):
    fan_present = 'gpio174'
    name = 'Fan6'
    driver_hwmon = ['fan7']

class QuantaLY6PSU1(QuantaLY6_PSU):
    name = 'PSU1'
    psu_present = 'gpio184'
    psu_ac_ok = 'gpio190'
    psu_dc_ok = 'gpio185'

class QuantaLY6PSU2(QuantaLY6_PSU):
    name = 'PSU2'
    psu_present = 'gpio187'
    psu_ac_ok = 'gpio191'
    psu_dc_ok = 'gpio188'

class QuantaLY6_LED(cumulus.sensors.StatusLED_Unit):
    good_led_color = '1'  # Green
    bad_led_color = '0'   # Off

    def led_update(self, color):
        self.set_gpio_value(self.led_location, color)
        #print 'loc: %s color %s' %(self.led_location, color)

    def led_read(self):
        color = self.get_gpio_value(self.led_location)
        if color == '1':
            color = 'green'
        else:
            color = 'off'
        return self.name, color

class QuantaLY6PSU1LED(QuantaLY6_LED):
    name = 'PSU1'
    sources = ['PSU1']
    led_location = 'gpio194'

class QuantaLY6PSU2LED(QuantaLY6_LED):
    name = 'PSU2'
    sources = ['PSU2']
    led_location = 'gpio196'

class QuantaLY6FanLED(QuantaLY6_LED):
    name = 'Fan'
    sources = ['Fan1', 'Fan2', 'Fan3', 'Fan4', 'Fan5', 'Fan6']
    led_location = 'gpio198'

class QuantaLY6SystemLED(QuantaLY6_LED):
    name = 'System'
    led_location = 'gpio235'

class QuantaLY6Temp(cumulus.sensors.Temp_Unit):
    temp_max = 75

class QuantaLY6Temp1(QuantaLY6Temp):
    name = 'Temp1'
    description = "Right Corner near PHY"
    driver_path = '/sys/devices/soc.0/ffe03000.i2c/i2c-0/0-004e'
    driver_hwmon = ['temp1']

class QuantaLY6Temp2(QuantaLY6Temp):
    name = 'Temp2'
    description = cumulus.sensors.Temp_Unit.mac_board_sensor_name
    driver_path = '/sys/devices/soc.0/ffe03000.i2c/i2c-0/0-004e'
    driver_hwmon = ['temp2']

class QuantaLY6Temp3(QuantaLY6Temp):
    name = 'Temp3'
    description = "Near Fan Outlet"
    driver_path = '/sys/devices/soc.0/ffe03000.i2c/i2c-0/0-004e'
    driver_hwmon = ['temp3']

class QuantaLY6Temp4(QuantaLY6Temp):
    name = 'Temp4'
    description = "Near CPU on CPU Board"
    driver_path = '/sys/devices/soc.0/ffe03000.i2c/i2c-0/0-004e'
    driver_hwmon = ['temp5']

class QuantaLY6Temp5(QuantaLY6Temp):
    name = 'Temp5'
    description = "Left Top Corner on CPU Board"
    driver_path = '/sys/devices/soc.0/ffe03000.i2c/i2c-0/0-004e'
    driver_hwmon = ['temp6']

class QuantaLY6PWM1(cumulus.sensors.PWM_Unit):
    fans = (QuantaLY6Fan1(),
            QuantaLY6Fan2(),
            QuantaLY6Fan3(),
            QuantaLY6Fan4(),
            QuantaLY6Fan5(),
            QuantaLY6Fan6(),
            )
    temps = (QuantaLY6Temp1(),
             QuantaLY6Temp2(),
             QuantaLY6Temp3(),
             QuantaLY6Temp4(),
             QuantaLY6Temp5(),
             )

class QuantaLY6Sensors():
    units = ( QuantaLY6PSU1(),
              QuantaLY6PSU2(),
              QuantaLY6Fan1(),
              QuantaLY6Fan2(),
              QuantaLY6Fan3(),
              QuantaLY6Fan4(),
              QuantaLY6Fan5(),
              QuantaLY6Fan6(),
              QuantaLY6Temp1(),
              QuantaLY6Temp2(),
              QuantaLY6Temp3(),
              QuantaLY6Temp4(),
              QuantaLY6Temp5(),
              )

class QuantaLY6PWMs():
    units = (QuantaLY6PWM1(),
             )

class QuantaLY6LEDs():
    units = (QuantaLY6PSU1LED(),
             QuantaLY6PSU2LED(),
             QuantaLY6SystemLED(),
             QuantaLY6FanLED(),
             )

class QuantaLY6P2020(cumulus.platform.Platform):
    name = 'quanta,ly6_p2020'
    chip = cumulus.platform.TridentTwo_56850_Chip()
    switch = cumulus.platforms.quanta_ly6.QuantaLY6Switch(chip)
    statusleds = QuantaLY6LEDs()
    pwms = QuantaLY6PWMs()
    sensors = QuantaLY6Sensors()

Platform = QuantaLY6P2020
