#
# Copyright 2016.  Cumulus Networks, Inc.
#
import os

import cumulus.platform
import cumulus.cpld
import cumulus.ledup
import cumulus.sunit
import cumulus.sfp
import cumulus.sensors
import cumulus.platforms.quanta_ly6

class QuantaLY6RangeleyFan(cumulus.sensors.Fan_Unit):
    driver_path = ''
    pci_path = '/sys/devices/pci0000:00/0000:00:1f.3'
    if os.path.exists(pci_path):
        for x in os.listdir(pci_path):
            if x.startswith('i2c-'):
                a,b = x.split('-')
                c = b + '-004e'
                driver_path = os.path.join(pci_path, x, c)                
                break

    fan_min = 9400
    fan_max = 16500
    rpmmode = 1

    def find_and_add_attrs(self):
        if self.get_gpio_value(self.fan_present) == '0':
            self.attrs['fan_0'] = 'installed, ok'
        else:
            self.attrs['fan_0'] = 'not_installed'
        return cumulus.sensors.Fan_Unit.find_and_add_attrs(self)

class QuantaLY6RangeleyCoreTemp(cumulus.sensors.Temp_Unit):
    description = cumulus.sensors.Temp_Unit.intel_cpu_sensor_name
    driver_path = '/sys/devices/platform/coretemp.0'
    temp_fan_min = 65
    temp_max = 90
    temp_crit = 98

class QuantaLY6RangeleyCY8Temp(cumulus.sensors.Temp_Unit):
    driver_path = ''
    pci_path = '/sys/devices/pci0000:00/0000:00:1f.3'
    if os.path.exists(pci_path):
        for x in os.listdir(pci_path):
            if x.startswith('i2c-'):
                a,b = x.split('-')
                c = b + '-004e'
                driver_path = os.path.join(pci_path, x, c)
                break

    temp_fan_min = 45
    temp_max = 70
    # OTP Temp is 80C as per Quanta
    temp_crit = 80

class QuantaLY6RangeleyPSU(cumulus.sensors.PSU_Unit):
    def find_and_add_attrs(self):
        if self.get_gpio_value(self.psu_present) == '0':
            self.attrs['psu_pwr1_present'] = 1
        else:
            self.attrs['psu_pwr1_present'] = 0
        if ((self.get_gpio_value(self.psu_ac_ok) == '1') and
            (self.get_gpio_value(self.psu_dc_ok) == '1')):
            self.attrs['psu_pwr1_all_ok'] = 1
        else:
            self.attrs['psu_pwr1_all_ok'] = 0
        return 0

class QuantaLY6RangeleyLED(cumulus.sensors.StatusLED_Unit):
    good_led_color = '1'  # Green
    bad_led_color = '0'   # Off

    def led_update(self, color):
        #print 'loc: %s color %s' %(self.led_location, color)
        self.set_gpio_value(self.led_location, color)

    def led_read(self):
        color = self.get_gpio_value(self.led_location)
        if color == '0':
            color = 'off'
        else:
            color = 'green'
        return self.name, color

class QuantaLY6RangeleyFan1(QuantaLY6RangeleyFan):
    fan_present = 'fan1_present'
    name = 'Fan1'
    description = 'Fan Tray 1'
    driver_hwmon = ['fan1']

class QuantaLY6RangeleyFan2(QuantaLY6RangeleyFan):
    fan_present = 'fan1_present'
    name = 'Fan2'
    description = 'Fan Tray 1'
    driver_hwmon = ['fan5']

class QuantaLY6RangeleyFan3(QuantaLY6RangeleyFan):
    fan_present = 'fan2_present'
    name = 'Fan3'
    description = 'Fan Tray 2'
    driver_hwmon = ['fan2']

class QuantaLY6RangeleyFan4(QuantaLY6RangeleyFan):
    fan_present = 'fan2_present'
    name = 'Fan4'
    description = 'Fan Tray 2'
    driver_hwmon = ['fan6']

class QuantaLY6RangeleyFan5(QuantaLY6RangeleyFan):
    fan_present = 'fan3_present'
    name = 'Fan5'
    description = 'Fan Tray 3'
    driver_hwmon = ['fan3']

class QuantaLY6RangeleyFan6(QuantaLY6RangeleyFan):
    fan_present = 'fan3_present'
    name = 'Fan6'
    description = 'Fan Tray 3'
    driver_hwmon = ['fan7']

class QuantaLY6RangeleyPSU1(QuantaLY6RangeleyPSU):
    name = 'PSU1'
    psu_present = 'psu_pwr1_present'
    psu_ac_ok = 'psu_pwr1_ac_ok'
    psu_dc_ok = 'psu_pwr1_dc_ok'

class QuantaLY6RangeleyPSU2(QuantaLY6RangeleyPSU):
    name = 'PSU2'
    psu_present = 'psu_pwr2_present'
    psu_ac_ok = 'psu_pwr2_ac_ok'
    psu_dc_ok = 'psu_pwr2_dc_ok'

class QuantaLY6RangeleyTemp1(QuantaLY6RangeleyCoreTemp):
    name = 'Temp1'
    driver_hwmon = ['temp2']

class QuantaLY6RangeleyTemp2(QuantaLY6RangeleyCoreTemp):
    name = 'Temp2'
    driver_hwmon = ['temp3']

class QuantaLY6RangeleyTemp3(QuantaLY6RangeleyCoreTemp):
    name = 'Temp3'
    driver_hwmon = ['temp4']

class QuantaLY6RangeleyTemp4(QuantaLY6RangeleyCoreTemp):
    name = 'Temp4'
    driver_hwmon = ['temp5']

class QuantaLY6RangeleyTemp5(QuantaLY6RangeleyCoreTemp):
    name = 'Temp5'
    driver_hwmon = ['temp6']

class QuantaLY6RangeleyTemp6(QuantaLY6RangeleyCoreTemp):
    name = 'Temp6'
    driver_hwmon = ['temp7']

class QuantaLY6RangeleyTemp7(QuantaLY6RangeleyCoreTemp):
    name = 'Temp7'
    driver_hwmon = ['temp8']

class QuantaLY6RangeleyTemp8(QuantaLY6RangeleyCoreTemp):
    name = 'Temp8'
    driver_hwmon = ['temp9']

class QuantaLY6RangeleyTemp9(QuantaLY6RangeleyCY8Temp):
    name = 'Temp9'
    description = "Right Corner near PHY"
    driver_hwmon = ['temp1']

class QuantaLY6RangeleyTemp10(QuantaLY6RangeleyCY8Temp):
    name = 'Temp10'
    description = cumulus.sensors.Temp_Unit.mac_board_sensor_name
    driver_hwmon = ['temp2']

class QuantaLY6RangeleyTemp11(QuantaLY6RangeleyCY8Temp):
    name = 'Temp11'
    description = "Near Fan Outlet"
    driver_hwmon = ['temp3']

class QuantaLY6RangeleyTemp12(QuantaLY6RangeleyCY8Temp):
    name = 'Temp12'
    description = "Near CPU on CPU Board"
    driver_hwmon = ['temp5']

class QuantaLY6RangeleyTemp13(QuantaLY6RangeleyCY8Temp):
    name = 'Temp13'
    description = "Left Top Corner on CPU Board"
    driver_hwmon = ['temp6']

class QuantaLY6RangeleyTemp14(cumulus.sensors.Temp_Unit):
    name = 'Temp14'
    description = cumulus.sensors.Temp_Unit.dimm_sensor_name
    driver_hwmon = ['temp1']
    temp_fan_min = 65
    temp_max = 90
    temp_crit = 95

    driver_path = ''
    pci_path = '/sys/devices/pci0000:00/0000:00:1f.3'
    if os.path.exists(pci_path):
        for x in os.listdir(pci_path):
            if x.startswith('i2c-'):
                a,b = x.split('-')
                c = b + '-001a'
                driver_path = os.path.join(pci_path, x, c)
                break

class QuantaLY6RangeleyPSU1LED(QuantaLY6RangeleyLED):
    name = 'PSU1'
    sources = ['PSU1']
    led_location = 'psu1_green_led'

class QuantaLY6RangeleyPSU2LED(QuantaLY6RangeleyLED):
    name = 'PSU2'
    sources = ['PSU2']
    led_location = 'psu2_green_led'

class QuantaLY6RangeleyFanLED(QuantaLY6RangeleyLED):
    name = 'Fan'
    sources = ['Fan1', 'Fan2', 'Fan3', 'Fan4', 'Fan5', 'Fan6']
    led_location = 'fan_green_led'

class QuantaLY6RangeleySensors():
    units = (QuantaLY6RangeleyTemp1(),
             QuantaLY6RangeleyTemp2(),
             QuantaLY6RangeleyTemp3(),
             QuantaLY6RangeleyTemp4(),
             QuantaLY6RangeleyTemp5(),
             QuantaLY6RangeleyTemp6(),
             QuantaLY6RangeleyTemp7(),
             QuantaLY6RangeleyTemp8(),
             QuantaLY6RangeleyTemp9(),
             QuantaLY6RangeleyTemp10(),
             QuantaLY6RangeleyTemp11(),
             QuantaLY6RangeleyTemp12(),
             QuantaLY6RangeleyTemp13(),
             QuantaLY6RangeleyTemp14(),
             QuantaLY6RangeleyFan1(),
             QuantaLY6RangeleyFan2(),
             QuantaLY6RangeleyFan3(),
             QuantaLY6RangeleyFan4(),
             QuantaLY6RangeleyFan5(),
             QuantaLY6RangeleyFan6(),
             QuantaLY6RangeleyPSU1(),
             QuantaLY6RangeleyPSU2(),
             )

class QuantaLY6RangeleyPWM1(cumulus.sensors.PWM_Unit):
    name = 'System PWM'
    fans = (QuantaLY6RangeleyFan1(),
            QuantaLY6RangeleyFan2(),
            QuantaLY6RangeleyFan3(),
            QuantaLY6RangeleyFan4(),
            QuantaLY6RangeleyFan5(),
            QuantaLY6RangeleyFan6(),
            )
    temps = (QuantaLY6RangeleyTemp1(),
             QuantaLY6RangeleyTemp2(),
             QuantaLY6RangeleyTemp3(),
             QuantaLY6RangeleyTemp4(),
             QuantaLY6RangeleyTemp5(),
             QuantaLY6RangeleyTemp6(),
             QuantaLY6RangeleyTemp7(),
             QuantaLY6RangeleyTemp8(),
             QuantaLY6RangeleyTemp9(),
             QuantaLY6RangeleyTemp10(),
             QuantaLY6RangeleyTemp11(),
             QuantaLY6RangeleyTemp12(),
             QuantaLY6RangeleyTemp13(),
             QuantaLY6RangeleyTemp14(),
             )

class QuantaLY6RangeleyPWMs():
    units = (QuantaLY6RangeleyPWM1(),
             )

class QuantaLY6RangeleyLEDs():
    units = (QuantaLY6RangeleyPSU1LED(),
             QuantaLY6RangeleyPSU2LED(),
             QuantaLY6RangeleyFanLED(),
             )

class QuantaLY6Rangeley(cumulus.platform.Platform):
    name = 'quanta,ly6_rangeley'
    chip = cumulus.platform.TridentTwo_56850_Chip()
    switch = cumulus.platforms.quanta_ly6.QuantaLY6Switch(chip)
    statusleds = QuantaLY6RangeleyLEDs()
    pwms = QuantaLY6RangeleyPWMs()
    sensors = QuantaLY6RangeleySensors()

Platform = QuantaLY6Rangeley
