#! /usr/bin/python

# Copyright 2012 Cumulus Networks LLC, all rights reserved
# Copyright 2016,2017 Cumulus Networks, Inc.  All rights reserved

#############################################################################
#
# Cumulus support cleanup script
#
############################################################################


try:
	import sys
	import os
	import time
	import glob
	import datetime
	import shutil

except ImportError, e:
	raise ImportError(str(e) + "- required module not found")

supportDir = "/var/support/"

#
# Removes old support files
#
def cleanupSupportFiles(goodFileCount=14):
	retval = 0
	try:
		supportFileList = glob.glob('*_support_*.t*z')
	except EnvironmentError, e:
		print 'Error reading dir ' + supportDir + ' for support files : ' + str(e)
		return 1

	if len(supportFileList) <= goodFileCount:
		return retval

	supportFileList.sort(reverse=True)
# Let's keep the oldest support though
	for i in range(goodFileCount, len(supportFileList) - 1):
		try:
			print 'deleting file ' + supportFileList[i]
			os.unlink(supportFileList[i])
		except EnvironmentError, e:
			print 'Error unlinking file ' + supportFileList[i] + str(e)
			retval = 1
			continue
	return retval

# If we find cl_support directories, and they are more than an hour old, assume
# that a support timed out or the system crashed, and remove the whole
# directory tree.
def cleanupSupportDirs():
	retval = 0
	try:
		supportDirList = glob.glob('*_support_*[0-9]*_[0-9]*[0-9]')
	except EnvironmentError, e:
		print 'Error checking for support partial directories ' + str(e)
		return 1

	if len(supportDirList) < 1:
		return retval

	hour_ago = datetime.datetime.today() - datetime.timedelta(hours=1)
	for adir in supportDirList:
		try:
			dirtime = datetime.datetime.fromtimestamp(os.path.getmtime(adir))
			if os.path.isdir(adir) and dirtime < hour_ago:
				print 'Removing directory tree ' + supportDir +  adir + '.'
				shutil.rmtree(adir, ignore_errors=True)

		except EnvironmentError, e:
			print 'Error checking ' + adir + ' for support partial directories ' + str(e)
			retval = 1

	return retval

try:
	os.chdir(supportDir)
except EnvironmentError, e:
	print 'Unable to chdir to ' + supportDir + ' to cleanup files : ' + str(e)
	sys.exit(1)

errors = cleanupSupportFiles()
errors += cleanupSupportDirs()

# avoid really frequent runs by delaying before terminating.
time.sleep(60)
sys.exit(errors)
